@@ -55,21 +55,37 @@ char *get_backup_name(char *fname)
 /* simple backup creates a backup with a suffix in the same directory */
 static int make_simple_backup(char *fname)
 {
+	int rename_errno;
 	char *fnamebak = get_backup_name(fname);
 
 	if (!fnamebak)
 		return 0;
 
-	if (do_rename(fname, fnamebak) != 0) {
-		/* cygwin (at least version b19) reports EINVAL */
-		if (errno != ENOENT && errno != EINVAL) {
-			rsyserr(FERROR, errno,
-				"rename %s to backup %s", fname, fnamebak);
-			return 0;
+	while (1) {
+		if (do_rename(fname, fnamebak) == 0) {
+			if (verbose > 1) {
+				rprintf(FINFO, "backed up %s to %s\n",
+					safe_fname(fname),
+					safe_fname(fnamebak));
+			}
+			break;
 		}
-	} else if (verbose > 1) {
-		rprintf(FINFO, "backed up %s to %s\n", fname, fnamebak);
+		/* cygwin (at least version b19) reports EINVAL */
+		if (errno == ENOENT || errno == EINVAL)
+			break;
+
+		rename_errno = errno;
+		if (errno == EISDIR && do_rmdir(fnamebak) == 0)
+			continue;
+		if (errno == ENOTDIR && do_unlink(fnamebak) == 0)
+			continue;
+
+		rsyserr(FERROR, rename_errno, "rename %s to backup %s",
+			safe_fname(fname), safe_fname(fnamebak));
+		errno = rename_errno;
+		return 0;
 	}
+
 	return 1;
 }
 
@@ -162,21 +178,15 @@ static int keep_backup(char *fname)
 	int ret_code;
 
 	/* return if no file to keep */
-#if SUPPORT_LINKS
-	ret_code = do_lstat(fname, &st);
-#else
-	ret_code = do_stat(fname, &st);
-#endif
-	if (ret_code < 0)
+	if (do_lstat(fname, &st) < 0)
 		return 1;
 
-	if (!(file = make_file(fname, NULL, NO_EXCLUDES)))
+	if (!(file = make_file(fname, NULL, NO_FILTERS)))
 		return 1; /* the file could have disappeared */
 
 	if (!(buf = get_backup_name(fname)))
 		return 0;
 
-#ifdef HAVE_MKNOD
 	/* Check to see if this is a device file, or link */
 	if (IS_DEVICE(file->mode)) {
 		if (am_root && preserve_devices) {
@@ -188,13 +198,12 @@ static int keep_backup(char *fname)
 			} else if (verbose > 2) {
 				rprintf(FINFO,
 					"make_backup: DEVICE %s successful.\n",
-					fname);
+					safe_fname(fname));
 			}
 		}
 		kept = 1;
 		do_unlink(fname);
 	}
-#endif
 
 	if (!kept && S_ISDIR(file->mode)) {
 		/* make an empty directory */
@@ -213,7 +222,7 @@ static int keep_backup(char *fname)
 		kept = 1;
 	}
 
-#if SUPPORT_LINKS
+#ifdef SUPPORT_LINKS
 	if (!kept && preserve_links && S_ISLNK(file->mode)) {
 		if (safe_symlinks && unsafe_symlink(file->u.link, buf)) {
 			if (verbose) {
@@ -226,7 +235,7 @@ static int keep_backup(char *fname)
 		    && (errno != ENOENT || make_bak_dir(buf) < 0
 		     || do_symlink(file->u.link, buf) < 0)) {
 			rsyserr(FERROR, errno, "link %s -> \"%s\"",
-				full_fname(buf), file->u.link);
+				full_fname(buf), safe_fname(file->u.link));
 		}
 		do_unlink(fname);
 		kept = 1;
@@ -235,7 +244,7 @@ static int keep_backup(char *fname)
 
 	if (!kept && !S_ISREG(file->mode)) {
 		rprintf(FINFO, "make_bak: skipping non-regular file %s\n",
-			fname);
+			safe_fname(fname));
 		return 1;
 	}
 
@@ -243,7 +252,7 @@ static int keep_backup(char *fname)
 	if (!kept) {
 		if (robust_move(fname, buf) != 0) {
 			rsyserr(FERROR, errno, "keep_backup failed: %s -> \"%s\"",
-				full_fname(fname), buf);
+				full_fname(fname), safe_fname(buf));
 		} else if (st.st_nlink > 1) {
 			/* If someone has hard-linked the file into the backup
 			 * dir, rename() might return success but do nothing! */
@@ -253,8 +262,10 @@ static int keep_backup(char *fname)
 	set_perms(buf, file, NULL, 0);
 	free(file);
 
-	if (verbose > 1)
-		rprintf(FINFO, "backed up %s to %s\n", fname, buf);
+	if (verbose > 1) {
+		rprintf(FINFO, "backed up %s to %s\n",
+			safe_fname(fname), safe_fname(buf));
+	}
 	return 1;
 }
 
