@@ -27,32 +27,37 @@
 
 #include "rsync.h"
 
+extern int verbose;
+extern int list_only;
 extern int am_sender;
 extern int am_server;
 extern int am_daemon;
 extern int am_root;
-extern int module_id;
-extern int read_only;
-extern int verbose;
 extern int rsync_port;
-extern int kludge_around_eof;
+extern int kluge_around_eof;
 extern int daemon_over_rsh;
-extern int list_only;
 extern int sanitize_paths;
 extern int filesfrom_fd;
 extern int remote_protocol;
 extern int protocol_version;
 extern int io_timeout;
+extern int select_timeout;
 extern int orig_umask;
 extern int no_detach;
 extern int default_af_hint;
 extern char *bind_address;
-extern struct exclude_list_struct server_exclude_list;
-extern char *exclude_path_prefix;
+extern struct filter_list_struct server_filter_list;
 extern char *config_file;
 extern char *files_from;
 
 char *auth_user;
+int read_only = 0;
+int daemon_log_format_has_i = 0;
+int daemon_log_format_has_o_or_i = 0;
+int module_id = -1;
+
+/* Length of lp_path() string when in daemon mode & not chrooted, else 0. */
+unsigned int module_dirlen = 0;
 
 /**
  * Run a client connected to an rsyncd.  The alternative to this
@@ -82,7 +87,7 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 		return -1;
 	}
 
-	if ((p = strchr(host, '@')) != NULL) {
+	if ((p = strrchr(host, '@')) != NULL) {
 		user = host;
 		host = p+1;
 		*p = '\0';
@@ -108,7 +113,7 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 	char *p;
 
 	if (argc == 0 && !am_sender)
-		list_only = 1;
+		list_only |= 1;
 
 	if (*path == '/') {
 		rprintf(FERROR,
@@ -121,19 +126,6 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 	if (!user)
 		user = getenv("LOGNAME");
 
-	/* set daemon_over_rsh to false since we need to build the
-	 * true set of args passed through the rsh/ssh connection;
-	 * this is a no-op for direct-socket-connection mode */
-	daemon_over_rsh = 0;
-	server_options(sargs, &sargc);
-
-	sargs[sargc++] = ".";
-
-	if (path && *path)
-		sargs[sargc++] = path;
-
-	sargs[sargc] = NULL;
-
 	io_printf(f_out, "@RSYNCD: %d\n", protocol_version);
 
 	if (!read_line(f_in, line, sizeof line - 1)) {
@@ -150,6 +142,25 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 	if (protocol_version > remote_protocol)
 		protocol_version = remote_protocol;
 
+	if (list_only && protocol_version >= 29)
+		list_only |= 2;
+
+	/* set daemon_over_rsh to false since we need to build the
+	 * true set of args passed through the rsh/ssh connection;
+	 * this is a no-op for direct-socket-connection mode */
+	daemon_over_rsh = 0;
+	server_options(sargs, &sargc);
+
+	sargs[sargc++] = ".";
+
+	if (path && *path)
+		sargs[sargc++] = path;
+
+	sargs[sargc] = NULL;
+
+	if (verbose > 1)
+		print_child_argv(sargs);
+
 	p = strchr(path,'/');
 	if (p) *p = 0;
 	io_printf(f_out, "%s\n", path);
@@ -157,7 +168,7 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 
 	/* Old servers may just drop the connection here,
 	 rather than sending a proper EXIT command.  Yuck. */
-	kludge_around_eof = list_only && (protocol_version < 25);
+	kluge_around_eof = list_only && protocol_version < 25 ? 1 : 0;
 
 	while (1) {
 		if (!read_line(f_in, line, sizeof line - 1)) {
@@ -190,7 +201,7 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 			rprintf(FINFO,"%s\n", line);
 		}
 	}
-	kludge_around_eof = False;
+	kluge_around_eof = 0;
 
 	for (i = 0; i < sargc; i++) {
 		io_printf(f_out, "%s\n", sargs[i]);
@@ -246,13 +257,12 @@ static int rsync_module(int f_in, int f_out, int i)
 	if (!claim_connection(lp_lock_file(i), lp_max_connections(i))) {
 		if (errno) {
 			rsyserr(FLOG, errno, "failed to open lock file %s",
-				lp_lock_file(i));
-			io_printf(f_out, "@ERROR: failed to open lock file %s\n",
-				  lp_lock_file(i));
+				safe_fname(lp_lock_file(i)));
+			io_printf(f_out, "@ERROR: failed to open lock file\n");
 		} else {
 			rprintf(FLOG, "max connections (%d) reached\n",
 				lp_max_connections(i));
-			io_printf(f_out, "@ERROR: max connections (%d) reached - try again later\n",
+			io_printf(f_out, "@ERROR: max connections (%d) reached -- try again later\n",
 				lp_max_connections(i));
 		}
 		return -1;
@@ -269,6 +279,17 @@ static int rsync_module(int f_in, int f_out, int i)
 
 	module_id = i;
 
+	if (lp_read_only(i))
+		read_only = 1;
+
+	if (lp_transfer_logging(i)) {
+		if (log_format_has(lp_log_format(i), 'i'))
+			daemon_log_format_has_i = 1;
+		if (daemon_log_format_has_i
+		    || log_format_has(lp_log_format(i), 'o'))
+			daemon_log_format_has_o_or_i = 1;
+	}
+
 	am_root = (MY_UID() == 0);
 
 	if (am_root) {
@@ -300,26 +321,34 @@ static int rsync_module(int f_in, int f_out, int i)
 	/* TODO: Perhaps take a list of gids, and make them into the
 	 * supplementary groups. */
 
-	exclude_path_prefix = use_chroot? "" : lp_path(i);
-	if (*exclude_path_prefix == '/' && !exclude_path_prefix[1])
-		exclude_path_prefix = "";
+	if (use_chroot) {
+		module_dirlen = 0;
+		set_filter_dir("/", 1);
+	} else {
+		module_dirlen = strlen(lp_path(i));
+		set_filter_dir(lp_path(i), module_dirlen);
+	}
+
+	p = lp_filter(i);
+	parse_rule(&server_filter_list, p, MATCHFLG_WORD_SPLIT,
+		   XFLG_ANCHORED2ABS);
 
 	p = lp_include_from(i);
-	add_exclude_file(&server_exclude_list, p,
-			 XFLG_FATAL_ERRORS | XFLG_DEF_INCLUDE);
+	parse_filter_file(&server_filter_list, p, MATCHFLG_INCLUDE,
+	    XFLG_ANCHORED2ABS | XFLG_OLD_PREFIXES | XFLG_FATAL_ERRORS);
 
 	p = lp_include(i);
-	add_exclude(&server_exclude_list, p,
-		    XFLG_WORD_SPLIT | XFLG_DEF_INCLUDE);
+	parse_rule(&server_filter_list, p,
+		   MATCHFLG_INCLUDE | MATCHFLG_WORD_SPLIT,
+		   XFLG_ANCHORED2ABS | XFLG_OLD_PREFIXES);
 
 	p = lp_exclude_from(i);
-	add_exclude_file(&server_exclude_list, p,
-			 XFLG_FATAL_ERRORS);
+	parse_filter_file(&server_filter_list, p, 0,
+	    XFLG_ANCHORED2ABS | XFLG_OLD_PREFIXES | XFLG_FATAL_ERRORS);
 
 	p = lp_exclude(i);
-	add_exclude(&server_exclude_list, p, XFLG_WORD_SPLIT);
-
-	exclude_path_prefix = NULL;
+	parse_rule(&server_filter_list, p, MATCHFLG_WORD_SPLIT,
+		   XFLG_ANCHORED2ABS | XFLG_OLD_PREFIXES);
 
 	log_init();
 
@@ -337,20 +366,23 @@ static int rsync_module(int f_in, int f_out, int i)
 		 * in which case we fail.
 		 */
 		if (chroot(lp_path(i))) {
-			rsyserr(FLOG, errno, "chroot %s failed", lp_path(i));
+			rsyserr(FLOG, errno, "chroot %s failed",
+				safe_fname(lp_path(i)));
 			io_printf(f_out, "@ERROR: chroot failed\n");
 			return -1;
 		}
 
 		if (!push_dir("/")) {
-			rsyserr(FLOG, errno, "chdir %s failed\n", lp_path(i));
+			rsyserr(FLOG, errno, "chdir %s failed\n",
+				safe_fname(lp_path(i)));
 			io_printf(f_out, "@ERROR: chdir failed\n");
 			return -1;
 		}
 
 	} else {
 		if (!push_dir(lp_path(i))) {
-			rsyserr(FLOG, errno, "chdir %s failed\n", lp_path(i));
+			rsyserr(FLOG, errno, "chdir %s failed\n",
+				safe_fname(lp_path(i)));
 			io_printf(f_out, "@ERROR: chdir failed\n");
 			return -1;
 		}
@@ -429,6 +461,7 @@ static int rsync_module(int f_in, int f_out, int i)
 			start_glob = 1;
 	}
 
+	verbose = 0; /* future verbosity is controlled by client options */
 	argp = argv;
 	ret = parse_arguments(&argc, (const char ***) &argp, 0);
 
@@ -462,8 +495,24 @@ static int rsync_module(int f_in, int f_out, int i)
 		 * get the error back to the client.  This means getting
 		 * the protocol setup finished first in later versions. */
 		setup_protocol(f_out, f_in);
-		if (files_from && !am_sender && strcmp(files_from, "-") != 0)
-			write_byte(f_out, 0);
+		if (!am_sender) {
+			/* Since we failed in our option parsing, we may not
+			 * have finished parsing that the client sent us a
+			 * --files-from option, so look for it manually.
+			 * Without this, the socket would be in the wrong
+			 * state for the upcoming error message. */
+			if (!files_from) {
+				int i;
+				for (i = 0; i < argc; i++) {
+					if (strncmp(argv[i], "--files-from", 12) == 0) {
+						files_from = "";
+						break;
+					}
+				}
+			}
+			if (files_from)
+				write_byte(f_out, 0);
+		}
 		io_start_multiplex_out();
 	}
 
@@ -473,8 +522,11 @@ static int rsync_module(int f_in, int f_out, int i)
 		exit_cleanup(RERR_UNSUPPORTED);
 	}
 
-	if (lp_timeout(i))
+	if (lp_timeout(i)) {
 		io_timeout = lp_timeout(i);
+		if (io_timeout < select_timeout)
+			select_timeout = io_timeout;
+	}
 
 	start_server(f_in, f_out, argc, argp);
 
@@ -598,6 +650,11 @@ int daemon_main(void)
 	if (!lp_load(config_file, 1))
 		exit_cleanup(RERR_SYNTAX);
 
+	if (rsync_port == 0 && (rsync_port = lp_rsync_port()) == 0)
+		rsync_port = RSYNC_PORT;
+	if (bind_address == NULL && *lp_bind_address())
+		bind_address = lp_bind_address();
+
 	log_init();
 
 	rprintf(FLOG, "rsyncd version %s starting, listening on port %d\n",
@@ -614,7 +671,8 @@ int daemon_main(void)
 		if ((fd = do_open(lp_pid_file(), O_WRONLY|O_CREAT|O_TRUNC,
 					0666 & ~orig_umask)) == -1) {
 			cleanup_set_pid(0);
-			rsyserr(FLOG, errno, "failed to create pid file %s", pid_file);
+			rsyserr(FLOG, errno, "failed to create pid file %s",
+				safe_fname(pid_file));
 			exit_cleanup(RERR_FILEIO);
 		}
 		snprintf(pidbuf, sizeof pidbuf, "%ld\n", (long)pid);
