@@ -6,17 +6,17 @@
  * Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
  */
 
-/* 
+/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -36,7 +36,7 @@
  * 3) add it to the list of available functions (eg: using FN_GLOBAL_STRING())
  * 4) If it's a global then initialise it in init_globals. If a local
  *    (ie. service) parameter then initialise it in the sDefault structure
- *  
+ *
  *
  * Notes:
  *   The configuration file is processed sequentially for speed. It is NOT
@@ -61,7 +61,7 @@ typedef char pstring[1024];
 typedef enum
 {
 	P_BOOL,P_BOOLREV,P_CHAR,P_INTEGER,P_OCTAL,
-	P_STRING,P_GSTRING,P_ENUM,P_SEP
+	P_PATH,P_STRING,P_GSTRING,P_ENUM,P_SEP
 } parm_type;
 
 typedef enum
@@ -93,7 +93,7 @@ struct parm_struct
 #define iSERVICE(i) (*pSERVICE(i))
 #define LP_SNUM_OK(iService) (((iService) >= 0) && ((iService) < iNumServices))
 
-/* 
+/*
  * This structure describes global (ie., server-wide) parameters.
  */
 typedef struct
@@ -101,16 +101,18 @@ typedef struct
 	char *motd_file;
 	char *log_file;
 	char *pid_file;
-	int syslog_facility;
 	char *socket_options;
+	char *bind_address;
+	int syslog_facility;
+	int max_verbosity;
+	int rsync_port;
 } global;
 
 static global Globals;
 
 
-
-/* 
- * This structure describes a single service. 
+/*
+ * This structure describes a single service.
  */
 typedef struct
 {
@@ -119,6 +121,7 @@ typedef struct
 	char *comment;
 	char *lock_file;
 	BOOL read_only;
+	BOOL write_only;
 	BOOL list;
 	BOOL use_chroot;
 	BOOL transfer_logging;
@@ -130,6 +133,7 @@ typedef struct
 	char *auth_users;
 	char *secrets_file;
 	BOOL strict_modes;
+	char *filter;
 	char *exclude;
 	char *exclude_from;
 	char *include;
@@ -144,19 +148,20 @@ typedef struct
 
 
 /* This is a default service used to prime a services structure */
-static service sDefault = 
+static service sDefault =
 {
 	NULL,    /* name */
 	NULL,    /* path */
 	NULL,    /* comment */
 	DEFAULT_LOCK_FILE,    /* lock file */
 	True,    /* read only */
+	False,   /* write only */
 	True,    /* list */
 	True,    /* use chroot */
 	False,   /* transfer logging */
 	False,   /* ignore errors */
 	"nobody",/* uid */
-	
+
 	/* TODO: This causes problems on Debian, where it is called
 	 * "nogroup".  Debian patch this in their version of the
 	 * package, but it would be nice to be consistent.  Possibly
@@ -165,12 +170,13 @@ static service sDefault =
 	 * What is the best behaviour?  Perhaps always using (gid_t)
 	 * -2? */
 	"nobody",/* gid */
-	
+
 	NULL,    /* hosts allow */
 	NULL,    /* hosts deny */
 	NULL,    /* auth users */
 	NULL,    /* secrets file */
 	True,   /* strict modes */
+	NULL,    /* filter */
 	NULL,    /* exclude */
 	NULL,    /* exclude from */
 	NULL,    /* include */
@@ -222,7 +228,7 @@ static struct enum_list enum_facilities[] = {
 	{ LOG_NEWS, "news" },
 #endif
 #ifdef LOG_AUTH
-	{ LOG_AUTH, "security" },		
+	{ LOG_AUTH, "security" },
 #endif
 #ifdef LOG_SYSLOG
 	{ LOG_SYSLOG, "syslog" },
@@ -268,14 +274,18 @@ static struct parm_struct parm_table[] =
   {"socket options",   P_STRING,  P_GLOBAL, &Globals.socket_options,NULL,  0},
   {"log file",         P_STRING,  P_GLOBAL, &Globals.log_file,      NULL,  0},
   {"pid file",         P_STRING,  P_GLOBAL, &Globals.pid_file,      NULL,  0},
+  {"max verbosity",    P_INTEGER, P_GLOBAL, &Globals.max_verbosity, NULL,  0},
+  {"port",             P_INTEGER, P_GLOBAL, &Globals.rsync_port,    NULL,  0},
+  {"address",          P_STRING,  P_GLOBAL, &Globals.bind_address,  NULL,  0},
 
   {"timeout",          P_INTEGER, P_LOCAL,  &sDefault.timeout,     NULL,  0},
   {"max connections",  P_INTEGER, P_LOCAL,  &sDefault.max_connections,NULL, 0},
   {"name",             P_STRING,  P_LOCAL,  &sDefault.name,        NULL,   0},
   {"comment",          P_STRING,  P_LOCAL,  &sDefault.comment,     NULL,   0},
   {"lock file",        P_STRING,  P_LOCAL,  &sDefault.lock_file,   NULL,   0},
-  {"path",             P_STRING,  P_LOCAL,  &sDefault.path,        NULL,   0},
+  {"path",             P_PATH,    P_LOCAL,  &sDefault.path,        NULL,   0},
   {"read only",        P_BOOL,    P_LOCAL,  &sDefault.read_only,   NULL,   0},
+  {"write only",       P_BOOL,    P_LOCAL,  &sDefault.write_only,  NULL,   0},
   {"list",             P_BOOL,    P_LOCAL,  &sDefault.list,        NULL,   0},
   {"use chroot",       P_BOOL,    P_LOCAL,  &sDefault.use_chroot,  NULL,   0},
   {"ignore nonreadable",P_BOOL,   P_LOCAL,  &sDefault.ignore_nonreadable,  NULL,   0},
@@ -286,6 +296,7 @@ static struct parm_struct parm_table[] =
   {"auth users",       P_STRING,  P_LOCAL,  &sDefault.auth_users,  NULL,   0},
   {"secrets file",     P_STRING,  P_LOCAL,  &sDefault.secrets_file,NULL,   0},
   {"strict modes",     P_BOOL,    P_LOCAL,  &sDefault.strict_modes,NULL,   0},
+  {"filter",           P_STRING,  P_LOCAL,  &sDefault.filter,      NULL,   0},
   {"exclude",          P_STRING,  P_LOCAL,  &sDefault.exclude,     NULL,   0},
   {"exclude from",     P_STRING,  P_LOCAL,  &sDefault.exclude_from,NULL,   0},
   {"include",          P_STRING,  P_LOCAL,  &sDefault.include,     NULL,   0},
@@ -304,10 +315,11 @@ Initialise the global parameter structure.
 ***************************************************************************/
 static void init_globals(void)
 {
-	memset(&Globals, 0, sizeof(Globals));
+	memset(&Globals, 0, sizeof Globals);
 #ifdef LOG_DAEMON
 	Globals.syslog_facility = LOG_DAEMON;
 #endif
+	Globals.max_verbosity = 1;
 }
 
 /***************************************************************************
@@ -319,8 +331,8 @@ static void init_locals(void)
 
 
 /*
-   In this section all the functions that are used to access the 
-   parameters from the rest of the program are defined 
+   In this section all the functions that are used to access the
+   parameters from the rest of the program are defined
 */
 
 #define FN_GLOBAL_STRING(fn_name,ptr) \
@@ -347,12 +359,16 @@ FN_GLOBAL_STRING(lp_log_file, &Globals.log_file)
 FN_GLOBAL_STRING(lp_pid_file, &Globals.pid_file)
 FN_GLOBAL_STRING(lp_socket_options, &Globals.socket_options)
 FN_GLOBAL_INTEGER(lp_syslog_facility, &Globals.syslog_facility)
+FN_GLOBAL_INTEGER(lp_max_verbosity, &Globals.max_verbosity)
+FN_GLOBAL_INTEGER(lp_rsync_port, &Globals.rsync_port)
+FN_GLOBAL_STRING(lp_bind_address, &Globals.bind_address)
 
 FN_LOCAL_STRING(lp_name, name)
 FN_LOCAL_STRING(lp_comment, comment)
 FN_LOCAL_STRING(lp_path, path)
 FN_LOCAL_STRING(lp_lock_file, lock_file)
 FN_LOCAL_BOOL(lp_read_only, read_only)
+FN_LOCAL_BOOL(lp_write_only, write_only)
 FN_LOCAL_BOOL(lp_list, list)
 FN_LOCAL_BOOL(lp_use_chroot, use_chroot)
 FN_LOCAL_BOOL(lp_transfer_logging, transfer_logging)
@@ -365,6 +381,7 @@ FN_LOCAL_STRING(lp_hosts_deny, hosts_deny)
 FN_LOCAL_STRING(lp_auth_users, auth_users)
 FN_LOCAL_STRING(lp_secrets_file, secrets_file)
 FN_LOCAL_BOOL(lp_strict_modes, strict_modes)
+FN_LOCAL_STRING(lp_filter, filter)
 FN_LOCAL_STRING(lp_exclude, exclude)
 FN_LOCAL_STRING(lp_exclude_from, exclude_from)
 FN_LOCAL_STRING(lp_include, include)
@@ -376,12 +393,11 @@ FN_LOCAL_INTEGER(lp_timeout, timeout)
 FN_LOCAL_INTEGER(lp_max_connections, max_connections)
 
 /* local prototypes */
-static int    strwicmp( char *psz1, char *psz2 );
-static int    map_parameter( char *parmname);
-static BOOL   set_boolean( BOOL *pb, char *parmvalue );
+static int    strwicmp(char *psz1, char *psz2);
+static int    map_parameter(char *parmname);
+static BOOL   set_boolean(BOOL *pb, char *parmvalue);
 static int    getservicebyname(char *name, service *pserviceDest);
-static void   copy_service( service *pserviceDest, 
-                            service *pserviceSource);
+static void   copy_service(service *pserviceDest, service *pserviceSource);
 static BOOL   do_parameter(char *parmname, char *parmvalue);
 static BOOL   do_section(char *sectionname);
 
@@ -421,7 +437,7 @@ static void string_set(char **s, const char *v)
 
 
 /***************************************************************************
-add a new service to the services array initialising it with the given 
+add a new service to the services array initialising it with the given
 service
 ***************************************************************************/
 static int add_a_service(service *pservice, char *name)
@@ -433,7 +449,7 @@ static int add_a_service(service *pservice, char *name)
   tservice = *pservice;
 
   /* it might already exist */
-  if (name) 
+  if (name)
     {
       i = getservicebyname(name,NULL);
       if (i >= 0)
@@ -455,7 +471,7 @@ static int add_a_service(service *pservice, char *name)
   init_service(pSERVICE(i));
   copy_service(pSERVICE(i),&tservice);
   if (name)
-    string_set(&iSERVICE(i).name,name);  
+    string_set(&iSERVICE(i).name,name);
 
   return(i);
 }
@@ -493,7 +509,7 @@ static int strwicmp(char *psz1, char *psz2)
 }
 
 /***************************************************************************
-Map a parameter's string representation to something we can use. 
+Map a parameter's string representation to something we can use.
 Returns False if the parameter string is not recognised, else TRUE.
 ***************************************************************************/
 static int map_parameter(char *parmname)
@@ -503,7 +519,7 @@ static int map_parameter(char *parmname)
    if (*parmname == '-')
      return(-1);
 
-   for (iIndex = 0; parm_table[iIndex].label; iIndex++) 
+   for (iIndex = 0; parm_table[iIndex].label; iIndex++)
       if (strwicmp(parm_table[iIndex].label, parmname) == 0)
          return(iIndex);
 
@@ -514,7 +530,7 @@ static int map_parameter(char *parmname)
 
 /***************************************************************************
 Set a boolean variable from the text value stored in the passed string.
-Returns True in success, False if the passed string does not correctly 
+Returns True in success, False if the passed string does not correctly
 represent a boolean.
 ***************************************************************************/
 static BOOL set_boolean(BOOL *pb, char *parmvalue)
@@ -548,7 +564,7 @@ static int getservicebyname(char *name, service *pserviceDest)
    int iService;
 
    for (iService = iNumServices - 1; iService >= 0; iService--)
-      if (strwicmp(iSERVICE(iService).name, name) == 0) 
+      if (strwicmp(iSERVICE(iService).name, name) == 0)
       {
          if (pserviceDest != NULL)
 	   copy_service(pserviceDest, pSERVICE(iService));
@@ -564,7 +580,7 @@ static int getservicebyname(char *name, service *pserviceDest)
 Copy a service structure to another
 
 ***************************************************************************/
-static void copy_service(service *pserviceDest, 
+static void copy_service(service *pserviceDest,
                          service *pserviceSource)
 {
   int i;
@@ -572,9 +588,9 @@ static void copy_service(service *pserviceDest,
   for (i=0;parm_table[i].label;i++)
     if (parm_table[i].ptr && parm_table[i].class == P_LOCAL) {
 	void *def_ptr = parm_table[i].ptr;
-	void *src_ptr = 
+	void *src_ptr =
 	  ((char *)pserviceSource) + PTR_DIFF(def_ptr,&sDefault);
-	void *dest_ptr = 
+	void *dest_ptr =
 	  ((char *)pserviceDest) + PTR_DIFF(def_ptr,&sDefault);
 
 	switch (parm_table[i].type)
@@ -594,6 +610,7 @@ static void copy_service(service *pserviceDest,
 	    *(char *)dest_ptr = *(char *)src_ptr;
 	    break;
 
+	  case P_PATH:
 	  case P_STRING:
 	    string_set(dest_ptr,*(char **)src_ptr);
 	    break;
@@ -614,6 +631,7 @@ static BOOL lp_do_parameter(int snum, char *parmname, char *parmvalue)
    int parmnum, i;
    void *parm_ptr=NULL; /* where we are going to store the result */
    void *def_ptr=NULL;
+   char *cp;
 
    parmnum = map_parameter(parmname);
 
@@ -660,6 +678,15 @@ static BOOL lp_do_parameter(int snum, char *parmname, char *parmvalue)
        sscanf(parmvalue,"%o",(int *)parm_ptr);
        break;
 
+     case P_PATH:
+       string_set(parm_ptr,parmvalue);
+       if ((cp = *(char**)parm_ptr) != NULL) {
+	   int len = strlen(cp);
+	   while (len > 1 && cp[len-1] == '/') len--;
+	   cp[len] = '\0';
+       }
+       break;
+
      case P_STRING:
        string_set(parm_ptr,parmvalue);
        break;
@@ -711,7 +738,7 @@ static BOOL do_section(char *sectionname)
      init_locals();
 
    /* if we've just struck a global section, note the fact. */
-   bInGlobalSection = isglobal;   
+   bInGlobalSection = isglobal;
 
    /* check for multiple global sections */
    if (bInGlobalSection)
@@ -743,7 +770,7 @@ static BOOL do_section(char *sectionname)
 
 
 /***************************************************************************
-Load the services array from the services file. Return True on success, 
+Load the services array from the services file. Return True on success,
 False on failure.
 ***************************************************************************/
 BOOL lp_load(char *pszFname, int globals_only)
@@ -753,11 +780,11 @@ BOOL lp_load(char *pszFname, int globals_only)
 	extern int am_root;
 	pstring n2;
 	BOOL bRetval;
- 
+
 	bRetval = False;
 
 	bInGlobalSection = True;
-  
+
 	init_globals();
 
 	if (pszFname)
@@ -770,7 +797,7 @@ BOOL lp_load(char *pszFname, int globals_only)
 	/* We get sections first, so have to start 'behind' to make up */
 	iServiceIndex = -1;
 	bRetval = pm_process(n2, globals_only?NULL:do_section, do_parameter);
-  
+
 	return (bRetval);
 }
 
@@ -794,7 +821,7 @@ int lp_number(char *name)
    int iService;
 
    for (iService = iNumServices - 1; iService >= 0; iService--)
-      if (strequal(lp_name(iService), name)) 
+      if (strcmp(lp_name(iService), name) == 0)
          break;
 
    return (iService);
