@@ -27,12 +27,19 @@
   */
 #include "rsync.h"
 
+extern int verbose;
+extern int dry_run;
 extern int am_daemon;
 extern int am_server;
 extern int am_sender;
+extern int local_server;
 extern int quiet;
 extern int module_id;
 extern int msg_fd_out;
+extern int protocol_version;
+extern int preserve_times;
+extern int log_format_has_o_or_i;
+extern int daemon_log_format_has_o_or_i;
 extern char *auth_user;
 extern char *log_format;
 
@@ -52,6 +59,7 @@ struct {
 	{ RERR_FILESELECT , "errors selecting input/output files, dirs" },
 	{ RERR_UNSUPPORTED, "requested action not supported" },
 	{ RERR_STARTCLIENT, "error starting client-server protocol" },
+	{ RERR_LOG_FAILURE, "daemon unable to append to log-file" },
 	{ RERR_SOCKETIO   , "error in socket IO" },
 	{ RERR_FILEIO     , "error in file IO" },
 	{ RERR_STREAMIO   , "error in rsync protocol data stream" },
@@ -65,8 +73,9 @@ struct {
 	{ RERR_TIMEOUT    , "timeout in data send/receive" },
 	{ RERR_CMD_FAILED , "remote shell failed" },
 	{ RERR_CMD_KILLED , "remote shell killed" },
-	{ RERR_CMD_RUN,     "remote command could not be run" },
-	{ RERR_CMD_NOTFOUND, "remote command not found" },
+	{ RERR_CMD_RUN    , "remote command could not be run" },
+	{ RERR_CMD_NOTFOUND,"remote command not found" },
+	{ RERR_DEL_LIMIT  , "the --max-delete limit stopped deletions" },
 	{ 0, NULL }
 };
 
@@ -146,6 +155,11 @@ void log_open(void)
 		int old_umask = umask(022 | orig_umask);
 		logfile = fopen(logfname, "a");
 		umask(old_umask);
+		if (!logfile) {
+			am_daemon = 0; /* avoid trying to log again */
+			rsyserr(FERROR, errno, "fopen() of log-file failed");
+			exit_cleanup(RERR_LOG_FAILURE);
+		}
 	}
 }
 
@@ -178,7 +192,9 @@ void rwrite(enum logcode code, char *buf, int len)
 		return;
 	}
 
-	if (am_daemon) {
+	if (code == FCLIENT)
+		code = FINFO;
+	else if (am_daemon) {
 		static int in_block;
 		char msg[2048];
 		int priority = code == FERROR ? LOG_WARNING : LOG_INFO;
@@ -224,7 +240,7 @@ void rwrite(enum logcode code, char *buf, int len)
 	if (buf[len-1] == '\r' || buf[len-1] == '\n')
 		fflush(f);
 }
-		
+
 
 /* This is the rsync debugging function. Call it with FINFO, FERROR or
  * FLOG. */
@@ -235,7 +251,7 @@ void rprintf(enum logcode code, const char *format, ...)
 	size_t len;
 
 	va_start(ap, format);
-	len = vsnprintf(buf, sizeof(buf), format, ap);
+	len = vsnprintf(buf, sizeof buf, format, ap);
 	va_end(ap);
 
 	/* Deal with buffer overruns.  Instead of panicking, just
@@ -245,20 +261,20 @@ void rprintf(enum logcode code, const char *format, ...)
 		const char ellipsis[] = "[...]";
 
 		/* Reset length, and zero-terminate the end of our buffer */
-		len = sizeof(buf)-1;
+		len = sizeof buf - 1;
 		buf[len] = '\0';
 
 		/* Copy the ellipsis to the end of the string, but give
 		 * us one extra character:
 		 *
-		 *                  v--- null byte at buf[sizeof(buf)-1]
+		 *                  v--- null byte at buf[sizeof buf - 1]
 		 *        abcdefghij0
 		 *     -> abcd[...]00  <-- now two null bytes at end
 		 *
 		 * If the input format string has a trailing newline,
 		 * we copy it into that extra null; if it doesn't, well,
 		 * all we lose is one byte.  */
-		strncpy(buf+len-sizeof(ellipsis), ellipsis, sizeof(ellipsis));
+		strncpy(buf+len-sizeof ellipsis, ellipsis, sizeof ellipsis);
 		if (format[strlen(format)-1] == '\n') {
 			buf[len-1] = '\n';
 		}
@@ -304,7 +320,7 @@ void rsyserr(enum logcode code, int errcode, const char *format, ...)
 void rflush(enum logcode code)
 {
 	FILE *f = NULL;
-	
+
 	if (am_daemon) {
 		return;
 	}
@@ -332,50 +348,96 @@ void rflush(enum logcode code)
 
 /* a generic logging routine for send/recv, with parameter
  * substitiution */
-static void log_formatted(enum logcode code,
-			  char *format, char *op, struct file_struct *file,
-			  struct stats *initial_stats)
+static void log_formatted(enum logcode code, char *format, char *op,
+			  struct file_struct *file, struct stats *initial_stats,
+			  int iflags, char *hlink)
 {
-	char buf[1024];
-	char buf2[1024];
+	char buf[MAXPATHLEN+1024], buf2[MAXPATHLEN], fmt[32];
 	char *p, *s, *n;
-	size_t l;
+	size_t len, total;
 	int64 b;
 
-	/* We expand % codes one by one in place in buf.  We don't
-	 * copy in the terminating nul of the inserted strings, but
-	 * rather keep going until we reach the nul of the format.
-	 * Just to make sure we don't clobber that nul and therefore
-	 * accidentally keep going, we zero the buffer now. */
-	l = strlcpy(buf, format, sizeof buf);
-	if (l < sizeof buf)
-		memset(buf + l, 0, sizeof buf - l);
-	
-	for (s = &buf[0]; s && (p = strchr(s,'%')); ) {
-		n = NULL;
-		s = p + 1;
+	*fmt = '%';
 
-		switch (p[1]) {
+	/* We expand % codes one by one in place in buf.  We don't
+	 * copy in the terminating null of the inserted strings, but
+	 * rather keep going until we reach the null of the format. */
+	total = strlcpy(buf, format, sizeof buf);
+	if (total > MAXPATHLEN) {
+		rprintf(FERROR, "log-format string is WAY too long!\n");
+		exit_cleanup(RERR_MESSAGEIO);
+	}
+	buf[total++] = '\n';
+	buf[total] = '\0';
+
+	for (p = buf; (p = strchr(p, '%')) != NULL; ) {
+		s = p++;
+		n = fmt + 1;
+		if (*p == '-')
+			*n++ = *p++;
+		while (isdigit(*(uchar*)p) && n - fmt < (int)(sizeof fmt) - 8)
+			*n++ = *p++;
+		if (!*p)
+			break;
+		*n = '\0';
+		n = NULL;
+
+		switch (*p) {
 		case 'h': if (am_daemon) n = client_name(0); break;
 		case 'a': if (am_daemon) n = client_addr(0); break;
 		case 'l':
-			snprintf(buf2,sizeof(buf2),"%.0f",
+			strlcat(fmt, ".0f", sizeof fmt);
+			snprintf(buf2, sizeof buf2, fmt,
 				 (double)file->length);
 			n = buf2;
 			break;
 		case 'p':
-			snprintf(buf2,sizeof(buf2),"%d",
-				 (int)getpid());
+			strlcat(fmt, "ld", sizeof fmt);
+			snprintf(buf2, sizeof buf2, fmt,
+				 (long)getpid());
 			n = buf2;
 			break;
 		case 'o': n = op; break;
 		case 'f':
-			pathjoin(buf2, sizeof buf2,
-				 file->basedir ? file->basedir : "",
-				 f_name(file));
-			clean_fname(buf2, 0);
+			n = safe_fname(f_name(file));
+			if (am_sender && file->dir.root) {
+				pathjoin(buf2, sizeof buf2,
+					 file->dir.root, n);
+				/* The buffer from safe_fname() has more
+				 * room than MAXPATHLEN, so this is safe. */
+				if (fmt[1])
+					strcpy(n, buf2);
+				else
+					n = buf2;
+			}
+			clean_fname(n, 0);
+			if (*n == '/')
+				n++;
+			break;
+		case 'n':
+			n = safe_fname(f_name(file));
+			if (S_ISDIR(file->mode)) {
+				/* The buffer from safe_fname() has more
+				 * room than MAXPATHLEN, so this is safe. */
+				strcat(n, "/");
+			}
+			break;
+		case 'L':
+			if (hlink && *hlink) {
+				n = safe_fname(hlink);
+				strcpy(buf2, " => ");
+			} else if (S_ISLNK(file->mode) && file->u.link) {
+				n = safe_fname(file->u.link);
+				strcpy(buf2, " -> ");
+			} else {
+				n = "";
+				if (!fmt[1])
+					break;
+				strcpy(buf2, "    ");
+			}
+			strlcat(fmt, "s", sizeof fmt);
+			snprintf(buf2 + 4, sizeof buf2 - 4, fmt, n);
 			n = buf2;
-			if (*n == '/') n++;
 			break;
 		case 'm': n = lp_name(module_id); break;
 		case 't': n = timestring(time(NULL)); break;
@@ -389,7 +451,8 @@ static void log_formatted(enum logcode code,
 				b = stats.total_read -
 					initial_stats->total_read;
 			}
-			snprintf(buf2,sizeof(buf2),"%.0f", (double)b);
+			strlcat(fmt, ".0f", sizeof fmt);
+			snprintf(buf2, sizeof buf2, fmt, (double)b);
 			n = buf2;
 			break;
 		case 'c':
@@ -400,62 +463,165 @@ static void log_formatted(enum logcode code,
 				b = stats.total_read -
 					initial_stats->total_read;
 			}
-			snprintf(buf2,sizeof(buf2),"%.0f", (double)b);
+			strlcat(fmt, ".0f", sizeof fmt);
+			snprintf(buf2, sizeof buf2, fmt, (double)b);
 			n = buf2;
 			break;
+		case 'i':
+			if (iflags & ITEM_DELETED) {
+				n = "*deleting";
+				break;
+			}
+			n = buf2 + MAXPATHLEN - 32;
+			n[0] = iflags & ITEM_LOCAL_CHANGE
+			      ? iflags & ITEM_XNAME_FOLLOWS ? 'h' : 'c'
+			     : !(iflags & ITEM_TRANSFER) ? '.'
+			     : !local_server && *op == 's' ? '<' : '>';
+			n[1] = S_ISDIR(file->mode) ? 'd'
+			     : IS_DEVICE(file->mode) ? 'D'
+			     : S_ISLNK(file->mode) ? 'L' : 'f';
+			n[2] = !(iflags & ITEM_REPORT_CHECKSUM) ? '.' : 'c';
+			n[3] = !(iflags & ITEM_REPORT_SIZE) ? '.' : 's';
+			n[4] = !(iflags & ITEM_REPORT_TIME) ? '.'
+			     : !preserve_times || IS_DEVICE(file->mode)
+					       || S_ISLNK(file->mode) ? 'T' : 't';
+			n[5] = !(iflags & ITEM_REPORT_PERMS) ? '.' : 'p';
+			n[6] = !(iflags & ITEM_REPORT_OWNER) ? '.' : 'o';
+			n[7] = !(iflags & ITEM_REPORT_GROUP) ? '.' : 'g';
+			n[8] = !(iflags & ITEM_REPORT_XATTRS) ? '.' : 'a';
+			n[9] = '\0';
+
+			if (iflags & (ITEM_IS_NEW|ITEM_MISSING_DATA)) {
+				char ch = iflags & ITEM_IS_NEW ? '+' : '?';
+				int i;
+				for (i = 2; n[i]; i++)
+					n[i] = ch;
+			} else if (!(iflags & (ITEM_TRANSFER|ITEM_LOCAL_CHANGE))) {
+				int i;
+				for (i = 2; n[i]; i++) {
+					if (n[i] != '.')
+						break;
+				}
+				if (!n[i]) {
+					for (i = 2; n[i]; i++)
+						n[i] = ' ';
+				}
+			}
+			break;
 		}
 
-		/* n is the string to be inserted in place of this %
-		 * code; l is its length not including the trailing
-		 * NUL */
+		/* "n" is the string to be inserted in place of this % code. */
 		if (!n)
 			continue;
+		if (n != buf2 && fmt[1]) {
+			strlcat(fmt, "s", sizeof fmt);
+			snprintf(buf2, sizeof buf2, fmt, n);
+			n = buf2;
+		}
+		len = strlen(n);
 
-		l = strlen(n);
+		/* Subtract the length of the escape from the string's size. */
+		total -= p - s + 1;
 
-		if (l + ((int)(s - &buf[0])) >= sizeof(buf)) {
-			rprintf(FERROR,"buffer overflow expanding %%%c - exiting\n",
+		if (len + total >= (size_t)sizeof buf) {
+			rprintf(FERROR,
+				"buffer overflow expanding %%%c -- exiting\n",
 				p[0]);
 			exit_cleanup(RERR_MESSAGEIO);
 		}
 
 		/* Shuffle the rest of the string along to make space for n */
-		if (l != 2) {
-			memmove(s+(l-1), s+1, strlen(s+1)+1);
-		}
+		if (len != (size_t)(p - s + 1))
+			memmove(s + len, p + 1, total - (s - buf) + 1);
+		total += len;
 
-		/* Copy in n but NOT its nul, because the format sting
-		 * probably continues after this. */
-		memcpy(p, n, l);
+		/* Insert the contents of string "n", but NOT its null. */
+		if (len)
+			memcpy(s, n, len);
 
 		/* Skip over inserted string; continue looking */
-		s = p+l;
+		p = s + len;
 	}
 
-	rprintf(code,"%s\n", buf);
+	rwrite(code, buf, total);
 }
 
-/* log the outgoing transfer of a file */
-void log_send(struct file_struct *file, struct stats *initial_stats)
+/* Return 1 if the format escape is in the log-format string (e.g. look for
+ * the 'b' in the "%9b" format escape). */
+int log_format_has(const char *format, char esc)
 {
-	if (lp_transfer_logging(module_id)) {
-		log_formatted(FLOG, lp_log_format(module_id), "send", file, initial_stats);
-	} else if (log_format && !am_server) {
-		log_formatted(FINFO, log_format, "send", file, initial_stats);
+	const char *p;
+
+	if (!format)
+		return 0;
+
+	for (p = format; (p = strchr(p, '%')) != NULL; ) {
+		if (*++p == '-')
+			p++;
+		while (isdigit(*(uchar*)p))
+			p++;
+		if (!*p)
+			break;
+		if (*p == esc)
+			return 1;
 	}
+	return 0;
 }
 
-/* log the incoming transfer of a file */
-void log_recv(struct file_struct *file, struct stats *initial_stats)
+/* log the transfer of a file */
+void log_item(struct file_struct *file, struct stats *initial_stats,
+	      int iflags, char *hlink)
 {
+	char *s_or_r = am_sender ? "send" : "recv";
+
 	if (lp_transfer_logging(module_id)) {
-		log_formatted(FLOG, lp_log_format(module_id), "recv", file, initial_stats);
+		log_formatted(FLOG, lp_log_format(module_id), s_or_r,
+			      file, initial_stats, iflags, hlink);
 	} else if (log_format && !am_server) {
-		log_formatted(FINFO, log_format, "recv", file, initial_stats);
+		log_formatted(FINFO, log_format, s_or_r,
+			      file, initial_stats, iflags, hlink);
 	}
 }
 
+void maybe_log_item(struct file_struct *file, int iflags, int itemizing,
+		    char *buf)
+{
+	int see_item = itemizing && (iflags || verbose > 1);
+	if (am_server) {
+		if (am_daemon && !dry_run && see_item)
+			log_item(file, &stats, iflags, buf);
+	} else if (see_item || iflags & ITEM_LOCAL_CHANGE || *buf
+	    || (S_ISDIR(file->mode) && iflags & SIGNIFICANT_ITEM_FLAGS))
+		log_item(file, &stats, iflags, buf);
+}
 
+void log_delete(char *fname, int mode)
+{
+	static struct file_struct file;
+	int len = strlen(fname);
+	char *fmt;
+
+	file.mode = mode;
+	file.basename = fname;
+
+	if (!verbose && !log_format)
+		;
+	else if (am_server && protocol_version >= 29 && len < MAXPATHLEN) {
+		if (S_ISDIR(mode))
+			len++; /* directories include trailing null */
+		send_msg(MSG_DELETED, fname, len);
+	} else {
+		fmt = log_format_has_o_or_i ? log_format : "deleting %n";
+		log_formatted(FCLIENT, fmt, "del.", &file, &stats,
+			      ITEM_DELETED, NULL);
+	}
+
+	if (!am_daemon || dry_run || !lp_transfer_logging(module_id))
+		return;
+
+	fmt = daemon_log_format_has_o_or_i ? lp_log_format(module_id) : "deleting %n";
+	log_formatted(FLOG, fmt, "del.", &file, &stats, ITEM_DELETED, NULL);
+}
 
 
 /*
@@ -467,7 +633,7 @@ void log_recv(struct file_struct *file, struct stats *initial_stats)
 void log_exit(int code, const char *file, int line)
 {
 	if (code == 0) {
-		rprintf(FLOG,"wrote %.0f bytes  read %.0f bytes  total size %.0f\n",
+		rprintf(FLOG,"sent %.0f bytes  received %.0f bytes  total size %.0f\n",
 			(double)stats.total_written,
 			(double)stats.total_read,
 			(double)stats.total_size);
