@@ -21,25 +21,20 @@
 
 #include "rsync.h"
 
-time_t starttime = 0;
-
-extern struct stats stats;
+extern int verbose;
+extern int dry_run;
+extern int list_only;
 extern int am_root;
 extern int am_server;
 extern int am_sender;
 extern int am_generator;
 extern int am_daemon;
-extern int verbose;
 extern int blocking_io;
-extern int cvs_exclude;
-extern int delete_mode;
-extern int delete_excluded;
-extern int delete_after;
+extern int remove_sent_files;
 extern int daemon_over_rsh;
+extern int need_messages_from_generator;
+extern int kluge_around_eof;
 extern int do_stats;
-extern int dry_run;
-extern int list_only;
-extern int local_server;
 extern int log_got_error;
 extern int module_id;
 extern int orig_umask;
@@ -48,8 +43,11 @@ extern int keep_dirlinks;
 extern int preserve_hard_links;
 extern int protocol_version;
 extern int recurse;
+extern int fuzzy_basis;
 extern int relative_paths;
 extern int rsync_port;
+extern int inplace;
+extern int make_backups;
 extern int whole_file;
 extern int read_batch;
 extern int write_batch;
@@ -57,16 +55,19 @@ extern int batch_fd;
 extern int batch_gen_fd;
 extern int filesfrom_fd;
 extern pid_t cleanup_child_pid;
-extern char *files_from;
-extern char *remote_filesfrom_file;
+extern struct stats stats;
+extern char *filesfrom_host;
+extern char *partial_dir;
+extern char *basis_dir[];
 extern char *rsync_path;
 extern char *shell_cmd;
 extern char *batch_name;
 
+int local_server = 0;
+struct file_list *the_file_list;
 
-/* there's probably never more than at most 2 outstanding child processes,
- * but set it higher just in case.
- */
+/* There's probably never more than at most 2 outstanding child processes,
+ * but set it higher, just in case. */
 #define MAXCHILDPROCS 5
 
 struct pid_status {
@@ -74,6 +75,9 @@ struct pid_status {
 	int   status;
 } pid_stat_table[MAXCHILDPROCS];
 
+static time_t starttime, endtime;
+static int64 total_read, total_written;
+
 static void show_malloc_stats(void);
 
 /****************************************************************************
@@ -118,12 +122,13 @@ void wait_process(pid_t pid, int *status)
  * the report.  All processes might also generate a set of debug stats, if
  * the verbose level is high enough (this is the only thing that the
  * generator process and the server receiver ever do here). */
-static void report(int f)
+static void handle_stats(int f)
 {
+	endtime = time(NULL);
+
 	/* Cache two stats because the read/write code can change it. */
-	int64 total_read = stats.total_read;
-	int64 total_written = stats.total_written;
-	time_t t = time(NULL);
+	total_read = stats.total_read;
+	total_written = stats.total_written;
 
 	if (do_stats && verbose > 1) {
 		/* These come out from every process */
@@ -145,6 +150,10 @@ static void report(int f)
 			write_longint(f, total_read);
 			write_longint(f, total_written);
 			write_longint(f, stats.total_size);
+			if (protocol_version >= 29) {
+				write_longint(f, stats.flist_buildtime);
+				write_longint(f, stats.flist_xfertime);
+			}
 		}
 		return;
 	}
@@ -157,12 +166,20 @@ static void report(int f)
 		total_written = read_longint(f);
 		total_read = read_longint(f);
 		stats.total_size = read_longint(f);
+		if (protocol_version >= 29) {
+			stats.flist_buildtime = read_longint(f);
+			stats.flist_xfertime = read_longint(f);
+		}
 	} else if (write_batch) {
 		/* The --read-batch process is going to be a client
 		 * receiver, so we need to give it the stats. */
 		write_longint(batch_fd, total_read);
 		write_longint(batch_fd, total_written);
 		write_longint(batch_fd, stats.total_size);
+		if (protocol_version >= 29) {
+			write_longint(batch_fd, stats.flist_buildtime);
+			write_longint(batch_fd, stats.flist_xfertime);
+		}
 	}
 
 	if (do_stats) {
@@ -178,17 +195,31 @@ static void report(int f)
 		rprintf(FINFO,"Matched data: %.0f bytes\n",
 			(double)stats.matched_data);
 		rprintf(FINFO,"File list size: %d\n", stats.flist_size);
+		if (stats.flist_buildtime) {
+			rprintf(FINFO,
+				"File list generation time: %.3f seconds\n",
+				(double)stats.flist_buildtime / 1000);
+			rprintf(FINFO,
+				"File list transfer time: %.3f seconds\n",
+				(double)stats.flist_xfertime / 1000);
+		}
 		rprintf(FINFO,"Total bytes sent: %.0f\n",
 			(double)total_written);
 		rprintf(FINFO,"Total bytes received: %.0f\n",
 			(double)total_read);
 	}
 
+	fflush(stdout);
+	fflush(stderr);
+}
+
+static void output_summary(void)
+{
 	if (verbose || do_stats) {
 		rprintf(FINFO,
 			"\nsent %.0f bytes  received %.0f bytes  %.2f bytes/sec\n",
 			(double)total_written, (double)total_read,
-			(total_written + total_read)/(0.5 + (t - starttime)));
+			(total_written + total_read)/(0.5 + (endtime - starttime)));
 		rprintf(FINFO, "total size is %.0f  speedup is %.2f\n",
 			(double)stats.total_size,
 			(double)stats.total_size / (total_written+total_read));
@@ -210,22 +241,30 @@ static void show_malloc_stats(void)
 	mi = mallinfo();
 
 	rprintf(FINFO, "\n" RSYNC_NAME "[%d] (%s%s%s) heap statistics:\n",
-		getpid(),
-		am_server ? "server " : "",
-		am_daemon ? "daemon " : "",
-		who_am_i());
-	rprintf(FINFO, "  arena:     %10d   (bytes from sbrk)\n", mi.arena);
-	rprintf(FINFO, "  ordblks:   %10d   (chunks not in use)\n", mi.ordblks);
-	rprintf(FINFO, "  smblks:    %10d\n", mi.smblks);
-	rprintf(FINFO, "  hblks:     %10d   (chunks from mmap)\n", mi.hblks);
-	rprintf(FINFO, "  hblkhd:    %10d   (bytes from mmap)\n", mi.hblkhd);
-	rprintf(FINFO, "  allmem:    %10d   (bytes from sbrk + mmap)\n",
-	    mi.arena + mi.hblkhd);
-	rprintf(FINFO, "  usmblks:   %10d\n", mi.usmblks);
-	rprintf(FINFO, "  fsmblks:   %10d\n", mi.fsmblks);
-	rprintf(FINFO, "  uordblks:  %10d   (bytes used)\n", mi.uordblks);
-	rprintf(FINFO, "  fordblks:  %10d   (bytes free)\n", mi.fordblks);
-	rprintf(FINFO, "  keepcost:  %10d   (bytes in releasable chunk)\n", mi.keepcost);
+		getpid(), am_server ? "server " : "",
+		am_daemon ? "daemon " : "", who_am_i());
+	rprintf(FINFO, "  arena:     %10ld   (bytes from sbrk)\n",
+		(long)mi.arena);
+	rprintf(FINFO, "  ordblks:   %10ld   (chunks not in use)\n",
+		(long)mi.ordblks);
+	rprintf(FINFO, "  smblks:    %10ld\n",
+		(long)mi.smblks);
+	rprintf(FINFO, "  hblks:     %10ld   (chunks from mmap)\n",
+		(long)mi.hblks);
+	rprintf(FINFO, "  hblkhd:    %10ld   (bytes from mmap)\n",
+		(long)mi.hblkhd);
+	rprintf(FINFO, "  allmem:    %10ld   (bytes from sbrk + mmap)\n",
+		(long)mi.arena + mi.hblkhd);
+	rprintf(FINFO, "  usmblks:   %10ld\n",
+		(long)mi.usmblks);
+	rprintf(FINFO, "  fsmblks:   %10ld\n",
+		(long)mi.fsmblks);
+	rprintf(FINFO, "  uordblks:  %10ld   (bytes used)\n",
+		(long)mi.uordblks);
+	rprintf(FINFO, "  fordblks:  %10ld   (bytes free)\n",
+		(long)mi.fordblks);
+	rprintf(FINFO, "  keepcost:  %10ld   (bytes in releasable chunk)\n",
+		(long)mi.keepcost);
 #endif /* HAVE_MALLINFO */
 }
 
@@ -266,7 +305,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char *path,
 				dash_l_set = 1;
 		}
 
-#if HAVE_REMSH
+#ifdef HAVE_REMSH
 		/* remsh (on HPUX) takes the arguments the other way around */
 		args[argc++] = machine;
 		if (user && !(daemon_over_rsh && dash_l_set)) {
@@ -311,7 +350,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char *path,
 	if (verbose > 3) {
 		rprintf(FINFO,"cmd=");
 		for (i = 0; i < argc; i++)
-			rprintf(FINFO,"%s ",args[i]);
+			rprintf(FINFO, "%s ", safe_fname(args[i]));
 		rprintf(FINFO,"\n");
 	}
 
@@ -381,7 +420,7 @@ static char *get_local_name(struct file_list *flist,char *name)
 		exit_cleanup(RERR_FILEIO);
 	}
 	if (verbose > 0)
-		rprintf(FINFO, "created directory %s\n", name);
+		rprintf(FINFO, "created directory %s\n", safe_fname(name));
 
 	if (dry_run) {
 		dry_run++;
@@ -398,6 +437,30 @@ static char *get_local_name(struct file_list *flist,char *name)
 }
 
 
+/* This is only called by the sender. */
+static void read_final_goodbye(int f_in, int f_out)
+{
+	int i;
+
+	if (protocol_version < 29)
+		i = read_int(f_in);
+	else {
+		while ((i = read_int(f_in)) == the_file_list->count
+		    && read_shortint(f_in) == ITEM_IS_NEW) {
+			/* Forward the keep-alive (no-op) to the receiver. */
+			write_int(f_out, the_file_list->count);
+			write_shortint(f_out, ITEM_IS_NEW);
+		}
+	}
+
+	if (i != -1) {
+		rprintf(FERROR, "Invalid packet at end of run (%d) [%s]\n",
+			i, who_am_i());
+		exit_cleanup(RERR_PROTOCOL);
+	}
+}
+
+
 static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
 {
 	int i;
@@ -414,6 +477,12 @@ static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
 		exit_cleanup(RERR_SYNTAX);
 		return;
 	}
+	if (am_daemon && lp_read_only(module_id) && remove_sent_files) {
+		rprintf(FERROR,
+		    "ERROR: --remove-sent-files cannot be used with a read-only module\n");
+		exit_cleanup(RERR_SYNTAX);
+		return;
+	}
 
 	if (!relative_paths && !push_dir(dir)) {
 		rsyserr(FERROR, errno, "push_dir#3 %s failed",
@@ -431,7 +500,7 @@ static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
 			argv[i] += l+1;
 	}
 
-	if (argc == 0 && recurse) {
+	if (argc == 0 && (recurse || list_only)) {
 		argc = 1;
 		argv--;
 		argv[0] = ".";
@@ -441,17 +510,16 @@ static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
 	if (!flist || flist->count == 0) {
 		exit_cleanup(0);
 	}
+	the_file_list = flist;
 
 	io_start_buffering_in();
 	io_start_buffering_out();
 
 	send_files(flist,f_out,f_in);
 	io_flush(FULL_FLUSH);
-	report(f_out);
-	if (protocol_version >= 24) {
-		/* final goodbye message */
-		read_int(f_in);
-	}
+	handle_stats(f_out);
+	if (protocol_version >= 24)
+		read_final_goodbye(f_in, f_out);
 	io_flush(FULL_FLUSH);
 	exit_cleanup(0);
 }
@@ -468,22 +536,21 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 	copy_links = 0;
 
 	if (preserve_hard_links)
-		init_hard_links(flist);
-
-	if (!delete_after) {
-		/* I moved this here from recv_files() to prevent a race condition */
-		if (recurse && delete_mode && !local_name && flist->count > 0)
-			delete_files(flist);
-	}
+		init_hard_links();
 
 	if (fd_pair(error_pipe) < 0) {
-		rprintf(FERROR,"error pipe failed in do_recv\n");
-		exit_cleanup(RERR_SOCKETIO);
+		rsyserr(FERROR, errno, "pipe failed in do_recv");
+		exit_cleanup(RERR_IPC);
 	}
 
 	io_flush(NORMAL_FLUSH);
 
-	if ((pid = do_fork()) == 0) {
+	if ((pid = do_fork()) == -1) {
+		rsyserr(FERROR, errno, "fork failed in do_recv");
+		exit_cleanup(RERR_IPC);
+	}
+
+	if (pid == 0) {
 		close(error_pipe[0]);
 		if (f_in != f_out)
 			close(f_out);
@@ -494,15 +561,30 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 		/* set place to send errors */
 		set_msg_fd_out(error_pipe[1]);
 
-		recv_files(f_in,flist,local_name);
+		recv_files(f_in, flist, local_name);
 		io_flush(FULL_FLUSH);
-		report(f_in);
+		handle_stats(f_in);
 
 		send_msg(MSG_DONE, "", 0);
 		io_flush(FULL_FLUSH);
-		/* finally we go to sleep until our parent kills us
-		 * with a USR2 signal. We sleep for a short time as on
-		 * some OSes a signal won't interrupt a sleep! */
+
+		/* Handle any keep-alive packets from the post-processing work
+		 * that the generator does. */
+		if (protocol_version >= 29) {
+			kluge_around_eof = -1;
+
+			/* This should only get stopped via a USR2 signal. */
+			while (read_int(f_in) == flist->count
+			    && read_shortint(f_in) == ITEM_IS_NEW) {}
+
+			rprintf(FERROR, "Invalid packet at end of run [%s]\n",
+				who_am_i());
+			exit_cleanup(RERR_PROTOCOL);
+		}
+
+		/* Finally, we go to sleep until our parent kills us with a
+		 * USR2 signal.  We sleep for a short time, as on some OSes
+		 * a signal won't interrupt a sleep! */
 		while (1)
 			msleep(20);
 	}
@@ -522,8 +604,7 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 
 	generate_files(f_out, flist, local_name);
 
-	get_redo_num(); /* Read final MSG_DONE and any prior messages. */
-	report(-1);
+	handle_stats(-1);
 	io_flush(FULL_FLUSH);
 	if (protocol_version >= 24) {
 		/* send a final goodbye message */
@@ -544,6 +625,13 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 	struct file_list *flist;
 	char *local_name = NULL;
 	char *dir = NULL;
+	int save_verbose = verbose;
+
+	if (filesfrom_fd >= 0) {
+		/* We can't mix messages with files-from data on the socket,
+		 * so temporarily turn off verbose messages. */
+		verbose = 0;
+	}
 
 	if (verbose > 2) {
 		rprintf(FINFO, "server_recv(%d) starting pid=%ld\n",
@@ -569,23 +657,25 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 	}
 
 	io_start_buffering_in();
-	if (delete_mode && !delete_excluded)
-		recv_exclude_list(f_in);
+	recv_filter_list(f_in);
 
 	if (filesfrom_fd >= 0) {
-		/* We're receiving the file info from the sender, so we need
-		 * the IO routines to automatically write out the names onto
-		 * our f_out socket as we read the list info from the sender.
-		 * This avoids both deadlock and extra delays/buffers. */
+		/* We need to send the files-from names to the sender at the
+		 * same time that we receive the file-list from them, so we
+		 * need the IO routines to automatically write out the names
+		 * onto our f_out socket as we read the file-list.  This
+		 * avoids both deadlock and extra delays/buffers. */
 		io_set_filesfrom_fds(filesfrom_fd, f_out);
 		filesfrom_fd = -1;
 	}
 
 	flist = recv_file_list(f_in);
+	verbose = save_verbose;
 	if (!flist) {
 		rprintf(FERROR,"server_recv: recv_file_list error\n");
 		exit_cleanup(RERR_FILESELECT);
 	}
+	the_file_list = flist;
 
 	if (argc > 0) {
 		if (strcmp(dir,".")) {
@@ -621,10 +711,10 @@ void start_server(int f_in, int f_out, int argc, char *argv[])
 
 	if (am_sender) {
 		keep_dirlinks = 0; /* Must be disabled on the sender. */
+		if (need_messages_from_generator)
+			io_start_multiplex_in();
 
-		recv_exclude_list(f_in);
-		if (cvs_exclude)
-			add_cvs_excludes();
+		recv_filter_list(f_in);
 		do_server_sender(f_in, f_out, argc, argv);
 	} else {
 		do_server_recv(f_in, f_out, argc, argv);
@@ -644,9 +734,7 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 	char *local_name = NULL;
 
 	cleanup_child_pid = pid;
-	if (read_batch) {
-		assert(am_sender == 0);
-	} else {
+	if (!read_batch) {
 		set_nonblocking(f_in);
 		set_nonblocking(f_out);
 	}
@@ -670,43 +758,44 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 	if (am_sender) {
 		keep_dirlinks = 0; /* Must be disabled on the sender. */
 		io_start_buffering_out();
-		if (cvs_exclude)
-			add_cvs_excludes();
-		if (delete_mode && !delete_excluded)
-			send_exclude_list(f_out);
-		if (remote_filesfrom_file)
+		if (!filesfrom_host)
+			set_msg_fd_in(f_in);
+		send_filter_list(f_out);
+		if (filesfrom_host)
 			filesfrom_fd = f_in;
 
 		if (write_batch)
 			start_write_batch(f_out);
-		if (!read_batch) /* don't write to pipe */
-			flist = send_file_list(f_out,argc,argv);
+		flist = send_file_list(f_out, argc, argv);
+		set_msg_fd_in(-1);
 		if (verbose > 3)
 			rprintf(FINFO,"file list sent\n");
+		the_file_list = flist;
 
 		io_flush(NORMAL_FLUSH);
 		send_files(flist,f_out,f_in);
 		io_flush(FULL_FLUSH);
-		if (protocol_version >= 24) {
-			/* final goodbye message */
-			read_int(f_in);
-		}
+		handle_stats(-1);
+		if (protocol_version >= 24)
+			read_final_goodbye(f_in, f_out);
 		if (pid != -1) {
 			if (verbose > 3)
 				rprintf(FINFO,"client_run waiting on %d\n", (int) pid);
 			io_flush(FULL_FLUSH);
 			wait_process(pid, &status);
 		}
-		report(-1);
+		output_summary();
 		io_flush(FULL_FLUSH);
 		exit_cleanup(status);
 	}
 
-	if (argc == 0)
-		list_only = 1;
+	if (need_messages_from_generator && !read_batch)
+		io_start_multiplex_out();
 
-	if (!read_batch)
-		send_exclude_list(f_out);
+	if (argc == 0)
+		list_only |= 1;
+
+	send_filter_list(read_batch ? -1 : f_out);
 
 	if (filesfrom_fd >= 0) {
 		io_set_filesfrom_fds(filesfrom_fd, f_out);
@@ -722,6 +811,7 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 			"the --recursive option?\n");
 		exit_cleanup(0);
 	}
+	the_file_list = flist;
 
 	local_name = get_local_name(flist,argv[0]);
 
@@ -777,134 +867,64 @@ static int start_client(int argc, char *argv[])
 	if ((rc = copy_argv(argv)))
 		return rc;
 
-	/* rsync:// always uses rsync server over direct socket connection */
-	if (strncasecmp(URL_PREFIX, argv[0], strlen(URL_PREFIX)) == 0
-	    && !read_batch) {
-		char *host, *path;
-
-		host = argv[0] + strlen(URL_PREFIX);
-		p = strchr(host,'/');
-		if (p) {
-			*p = '\0';
-			path = p+1;
-		} else
-			path = "";
-		if (*host == '[' && (p = strchr(host, ']')) != NULL) {
-			host++;
-			*p++ = '\0';
-			if (*p != ':')
-				p = NULL;
-		} else
-			p = strchr(host, ':');
-		if (p) {
-			rsync_port = atoi(p+1);
-			*p = '\0';
-		}
-		return start_socket_client(host, path, argc-1, argv+1);
-	}
-
 	if (!read_batch) { /* for read_batch, NO source is specified */
-		p = find_colon(argv[0]);
-		if (p) { /* source is remote */
-			if (remote_filesfrom_file
-			 && remote_filesfrom_file != files_from + 1
-			 && strncmp(files_from, argv[0], p-argv[0]+1) != 0) {
+		argc--;
+		shell_path = check_for_hostspec(argv[0], &shell_machine, &rsync_port);
+		if (shell_path) { /* source is remote */
+			argv++;
+			if (filesfrom_host && *filesfrom_host
+			    && strcmp(filesfrom_host, shell_machine) != 0) {
 				rprintf(FERROR,
 					"--files-from hostname is not the same as the transfer hostname\n");
 				exit_cleanup(RERR_SYNTAX);
 			}
-			if (p[1] == ':') { /* double colon */
-				*p = 0;
+			if (rsync_port) {
 				if (!shell_cmd) {
-					return start_socket_client(argv[0], p+2,
-								   argc-1, argv+1);
+					return start_socket_client(shell_machine,
+								   shell_path,
+								   argc, argv);
 				}
-				p++;
 				daemon_over_rsh = 1;
 			}
 
+			am_sender = 0;
+		} else { /* source is local, check dest arg */
+			am_sender = 1;
+
 			if (argc < 1) { /* destination required */
 				usage(FERROR);
 				exit_cleanup(RERR_SYNTAX);
 			}
 
-			am_sender = 0;
-			*p = 0;
-			shell_machine = argv[0];
-			shell_path = p+1;
-			argv++;
-		} else { /* source is local */
-			am_sender = 1;
-
-			/* rsync:// destination uses rsync server over direct socket */
-			if (strncasecmp(URL_PREFIX, argv[argc-1], strlen(URL_PREFIX)) == 0) {
-				char *host, *path;
-
-				host = argv[argc-1] + strlen(URL_PREFIX);
-				p = strchr(host,'/');
-				if (p) {
-					*p = '\0';
-					path = p+1;
-				} else
-					path = "";
-				if (*host == '[' && (p = strchr(host, ']')) != NULL) {
-					host++;
-					*p++ = '\0';
-					if (*p != ':')
-						p = NULL;
-				} else
-					p = strchr(host, ':');
-				if (p) {
-					rsync_port = atoi(p+1);
-					*p = '\0';
-				}
-				return start_socket_client(host, path, argc-1, argv);
-			}
-
-			p = find_colon(argv[argc-1]); /* look in dest arg */
-			if (p && remote_filesfrom_file
-			 && remote_filesfrom_file != files_from + 1
-			 && strncmp(files_from, argv[argc-1], p-argv[argc-1]+1) != 0) {
+			shell_path = check_for_hostspec(argv[argc], &shell_machine, &rsync_port);
+			if (shell_path && filesfrom_host && *filesfrom_host
+			    && strcmp(filesfrom_host, shell_machine) != 0) {
 				rprintf(FERROR,
 					"--files-from hostname is not the same as the transfer hostname\n");
 				exit_cleanup(RERR_SYNTAX);
 			}
-			if (!p) { /* no colon found, so src & dest are local */
+			if (!shell_path) { /* no hostspec found, so src & dest are local */
 				local_server = 1;
-				if (remote_filesfrom_file) {
+				if (filesfrom_host) {
 					rprintf(FERROR,
 						"--files-from cannot be remote when the transfer is local\n");
 					exit_cleanup(RERR_SYNTAX);
 				}
-			} else if (p[1] == ':') { /* double colon */
-				*p = 0;
+				shell_machine = NULL;
+				shell_path = argv[argc];
+			} else if (rsync_port) {
 				if (!shell_cmd) {
-					return start_socket_client(argv[argc-1], p+2,
-								   argc-1, argv);
+					return start_socket_client(shell_machine,
+								   shell_path,
+								   argc, argv);
 				}
-				p++;
 				daemon_over_rsh = 1;
 			}
-
-			if (argc < 2) {
-				usage(FERROR);
-				exit_cleanup(RERR_SYNTAX);
-			}
-
-			if (local_server) {
-				shell_machine = NULL;
-				shell_path = argv[argc-1];
-			} else {
-				*p = 0;
-				shell_machine = argv[argc-1];
-				shell_path = p+1;
-			}
 		}
-		argc--;
 	} else {  /* read_batch */
 		local_server = 1;
 		shell_path = argv[argc-1];
-		if (find_colon(shell_path)) {
+		if (check_for_hostspec(shell_path, &shell_machine, &rsync_port)) {
 			rprintf(FERROR, "remote destination is not allowed with --read-batch\n");
 			exit_cleanup(RERR_SYNTAX);
 		}
@@ -921,10 +941,10 @@ static int start_client(int argc, char *argv[])
 
 	if (verbose > 3) {
 		rprintf(FINFO,"cmd=%s machine=%s user=%s path=%s\n",
-			shell_cmd?shell_cmd:"",
-			shell_machine?shell_machine:"",
-			shell_user?shell_user:"",
-			shell_path?shell_path:"");
+			shell_cmd ? safe_fname(shell_cmd) : "",
+			shell_machine ? safe_fname(shell_machine) : "",
+			shell_user ? safe_fname(shell_user) : "",
+			shell_path ? safe_fname(shell_path) : "");
 	}
 
 	/* for remote source, only single dest arg can remain ... */
@@ -934,9 +954,8 @@ static int start_client(int argc, char *argv[])
 	}
 
 	/* ... or no dest at all */
-	if (!am_sender && argc == 0) {
-		list_only = 1;
-	}
+	if (!am_sender && argc == 0)
+		list_only |= 1;
 
 	pid = do_cmd(shell_cmd,shell_machine,shell_user,shell_path,
 		     &f_in,&f_out);
@@ -967,7 +986,11 @@ static RETSIGTYPE sigusr1_handler(UNUSED(int val))
 
 static RETSIGTYPE sigusr2_handler(UNUSED(int val))
 {
-	if (log_got_error) _exit(RERR_PARTIAL);
+	if (!am_server)
+		output_summary();
+	close_all();
+	if (log_got_error)
+		_exit(RERR_PARTIAL);
 	_exit(0);
 }
 
@@ -1113,7 +1136,7 @@ int main(int argc,char *argv[])
 		}
 		if (batch_fd < 0) {
 			rsyserr(FERROR, errno, "Batch file %s open error",
-				batch_name);
+				full_fname(batch_name));
 			exit_cleanup(RERR_FILEIO);
 		}
 		if (read_batch)
@@ -1128,9 +1151,6 @@ int main(int argc,char *argv[])
 		exit_cleanup(RERR_SYNTAX);
 	}
 
-	if (dry_run)
-		verbose = MAX(verbose,1);
-
 	if (am_server) {
 		set_nonblocking(STDIN_FILENO);
 		set_nonblocking(STDOUT_FILENO);
