@@ -21,122 +21,103 @@
 #include "rsync.h"
 
 extern int verbose;
-extern int recurse;
-extern int delete_after;
-extern int max_delete;
-extern int csum_length;
-extern struct stats stats;
 extern int dry_run;
+extern int am_daemon;
+extern int am_server;
+extern int do_progress;
+extern int log_before_transfer;
+extern int log_format_has_i;
+extern int daemon_log_format_has_i;
+extern int csum_length;
 extern int read_batch;
 extern int batch_gen_fd;
-extern int am_server;
+extern int protocol_version;
 extern int relative_paths;
 extern int keep_dirlinks;
 extern int preserve_hard_links;
 extern int preserve_perms;
-extern int cvs_exclude;
 extern int io_error;
-extern char *tmpdir;
-extern char *partial_dir;
-extern char *compare_dest;
+extern int basis_dir_cnt;
 extern int make_backups;
-extern int do_progress;
-extern char *backup_dir;
-extern char *backup_suffix;
-extern int backup_suffix_len;
 extern int cleanup_got_literal;
+extern int remove_sent_files;
 extern int module_id;
 extern int ignore_errors;
 extern int orig_umask;
 extern int keep_partial;
 extern int checksum_seed;
 extern int inplace;
+extern int delay_updates;
+extern struct stats stats;
+extern char *log_format;
+extern char *tmpdir;
+extern char *partial_dir;
+extern char *basis_dir[];
+extern struct file_list *the_file_list;
+extern struct filter_list_struct server_filter_list;
 
-extern struct exclude_list_struct server_exclude_list;
+#define SLOT_SIZE	(16*1024)	/* Desired size in bytes */
+#define PER_SLOT_BITS	(SLOT_SIZE * 8) /* Number of bits per slot */
+#define PER_SLOT_INTS	(SLOT_SIZE / 4) /* Number of int32s per slot */
 
+static uint32 **delayed_bits = NULL;
+static int delayed_slot_cnt = 0;
+static int phase = 0;
 
-static void delete_one(char *fn, int is_dir)
+static void init_delayed_bits(int max_ndx)
 {
-	if (!is_dir) {
-		if (robust_unlink(fn) != 0) {
-			rsyserr(FERROR, errno, "delete_one: unlink %s failed",
-				full_fname(fn));
-		} else if (verbose)
-			rprintf(FINFO, "deleting %s\n", safe_fname(fn));
-	} else {
-		if (do_rmdir(fn) != 0) {
-			if (errno != ENOTEMPTY && errno != EEXIST) {
-				rsyserr(FERROR, errno,
-					"delete_one: rmdir %s failed",
-					full_fname(fn));
-			}
-		} else if (verbose) {
-			rprintf(FINFO, "deleting directory %s\n",
-				safe_fname(fn));
-		}
-	}
+	delayed_slot_cnt = (max_ndx + PER_SLOT_BITS - 1) / PER_SLOT_BITS;
+
+	if (!(delayed_bits = (uint32**)calloc(delayed_slot_cnt, sizeof (uint32*))))
+		out_of_memory("set_delayed_bit");
 }
 
-
-static int is_backup_file(char *fn)
+static void set_delayed_bit(int ndx)
 {
-	int k = strlen(fn) - backup_suffix_len;
-	return k > 0 && strcmp(fn+k, backup_suffix) == 0;
-}
+	int slot = ndx / PER_SLOT_BITS;
+	ndx %= PER_SLOT_BITS;
 
-
-/* This deletes any files on the receiving side that are not present
- * on the sending side. */
-void delete_files(struct file_list *flist)
-{
-	struct file_list *local_file_list;
-	int i, j;
-	char *argv[1], fbuf[MAXPATHLEN];
-	static int deletion_count;
-
-	if (cvs_exclude)
-		add_cvs_excludes();
-
-	if (io_error && !(lp_ignore_errors(module_id) || ignore_errors)) {
-		rprintf(FINFO,"IO error encountered - skipping file deletion\n");
-		return;
+	if (!delayed_bits[slot]) {
+		if (!(delayed_bits[slot] = (uint32*)calloc(PER_SLOT_INTS, 4)))
+			out_of_memory("set_delayed_bit");
 	}
 
-	for (j = 0; j < flist->count; j++) {
-		if (!(flist->files[j]->flags & FLAG_TOP_DIR)
-		    || !S_ISDIR(flist->files[j]->mode))
+	delayed_bits[slot][ndx/32] |= 1u << (ndx % 32);
+}
+
+/* Call this with -1 to start checking from 0.  Returns -1 at the end. */
+static int next_delayed_bit(int after)
+{
+	uint32 bits, mask;
+	int i, ndx = after + 1;
+	int slot = ndx / PER_SLOT_BITS;
+	ndx %= PER_SLOT_BITS;
+
+	mask = (1u << (ndx % 32)) - 1;
+	for (i = ndx / 32; slot < delayed_slot_cnt; slot++, i = mask = 0) {
+		if (!delayed_bits[slot])
 			continue;
-
-		argv[0] = f_name_to(flist->files[j], fbuf);
-
-		if (!(local_file_list = send_file_list(-1, 1, argv)))
-			continue;
-
-		if (verbose > 1)
-			rprintf(FINFO, "deleting in %s\n", safe_fname(fbuf));
-
-		for (i = local_file_list->count-1; i >= 0; i--) {
-			if (max_delete && deletion_count > max_delete)
-				break;
-			if (!local_file_list->files[i]->basename)
+		for ( ; i < PER_SLOT_INTS; i++, mask = 0) {
+			if (!(bits = delayed_bits[slot][i] & ~mask))
 				continue;
-			if (flist_find(flist,local_file_list->files[i]) < 0) {
-				char *f = f_name(local_file_list->files[i]);
-				if (make_backups && (backup_dir || !is_backup_file(f))) {
-					make_backup(f);
-					if (verbose) {
-						rprintf(FINFO, "deleting %s\n",
-							safe_fname(f));
-					}
-				} else {
-					int mode = local_file_list->files[i]->mode;
-					delete_one(f, S_ISDIR(mode) != 0);
-				}
-				deletion_count++;
+			/* The xor magic figures out the lowest enabled bit in
+			 * bits, and the switch quickly computes log2(bit). */
+			switch (bits ^ (bits & (bits-1))) {
+#define LOG2(n) case 1u << n: return slot*PER_SLOT_BITS + i*32 + n
+			    LOG2(0);  LOG2(1);  LOG2(2);  LOG2(3);
+			    LOG2(4);  LOG2(5);  LOG2(6);  LOG2(7);
+			    LOG2(8);  LOG2(9);  LOG2(10); LOG2(11);
+			    LOG2(12); LOG2(13); LOG2(14); LOG2(15);
+			    LOG2(16); LOG2(17); LOG2(18); LOG2(19);
+			    LOG2(20); LOG2(21); LOG2(22); LOG2(23);
+			    LOG2(24); LOG2(25); LOG2(26); LOG2(27);
+			    LOG2(28); LOG2(29); LOG2(30); LOG2(31);
 			}
+			return -1; /* impossible... */
 		}
-		flist_free(local_file_list);
 	}
+
+	return -1;
 }
 
 
@@ -189,7 +170,8 @@ static int get_tmpname(char *fnametmp, char *fname)
 	maxname = MIN(MAXPATHLEN - 7 - length, NAME_MAX - 8);
 
 	if (maxname < 1) {
-		rprintf(FERROR, "temporary filename too long: %s\n", fname);
+		rprintf(FERROR, "temporary filename too long: %s\n",
+			safe_fname(fname));
 		fnametmp[0] = '\0';
 		return 0;
 	}
@@ -208,18 +190,18 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 	static char file_sum2[MD4_SUM_LENGTH];
 	struct map_struct *mapbuf;
 	struct sum_struct sum;
-	unsigned int len;
+	int32 len;
 	OFF_T offset = 0;
 	OFF_T offset2;
 	char *data;
-	int i;
+	int32 i;
 	char *map = NULL;
 
 	read_sum_head(f_in, &sum);
 
 	if (fd_r >= 0 && size_r > 0) {
-		OFF_T map_size = MAX(sum.blength * 2, 16*1024);
-		mapbuf = map_file(fd_r, size_r, map_size, sum.blength);
+		int32 read_size = MAX(sum.blength * 2, 16*1024);
+		mapbuf = map_file(fd_r, size_r, read_size, sum.blength);
 		if (verbose > 2) {
 			rprintf(FINFO, "recv mapped %s of size %.0f\n",
 				safe_fname(fname_r), (double)size_r);
@@ -242,7 +224,7 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 			stats.literal_data += i;
 			cleanup_got_literal = 1;
 
-			sum_update(data,i);
+			sum_update(data, i);
 
 			if (fd != -1 && write_file(fd,data,i) != i)
 				goto report_write_error;
@@ -251,22 +233,24 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 		}
 
 		i = -(i+1);
-		offset2 = i*(OFF_T)sum.blength;
+		offset2 = i * (OFF_T)sum.blength;
 		len = sum.blength;
 		if (i == (int)sum.count-1 && sum.remainder != 0)
 			len = sum.remainder;
 
 		stats.matched_data += len;
 
-		if (verbose > 3)
-			rprintf(FINFO,"chunk[%d] of size %d at %.0f offset=%.0f\n",
-				i,len,(double)offset2,(double)offset);
+		if (verbose > 3) {
+			rprintf(FINFO,
+				"chunk[%d] of size %ld at %.0f offset=%.0f\n",
+				i, (long)len, (double)offset2, (double)offset);
+		}
 
 		if (mapbuf) {
 			map = map_ptr(mapbuf,offset2,len);
 
 			see_token(map, len);
-			sum_update(map,len);
+			sum_update(map, len);
 		}
 
 		if (inplace) {
@@ -283,12 +267,13 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 				continue;
 			}
 		}
-		if (fd != -1 && write_file(fd, map, len) != (int)len)
+		if (fd != -1 && map && write_file(fd, map, len) != (int)len)
 			goto report_write_error;
 		offset += len;
 	}
 
-	flush_write_file(fd);
+	if (flush_write_file(fd) < 0)
+		goto report_write_error;
 
 #ifdef HAVE_FTRUNCATE
 	if (inplace && fd != -1)
@@ -324,6 +309,57 @@ static void discard_receive_data(int f_in, OFF_T length)
 	receive_data(f_in, NULL, -1, 0, NULL, -1, length);
 }
 
+static void handle_delayed_updates(struct file_list *flist, char *local_name)
+{
+	char *fname, *partialptr, numbuf[4];
+	int i;
+
+	for (i = -1; (i = next_delayed_bit(i)) >= 0; ) {
+		struct file_struct *file = flist->files[i];
+		fname = local_name ? local_name : f_name(file);
+		if ((partialptr = partial_dir_fname(fname)) != NULL) {
+			if (make_backups && !make_backup(fname))
+				continue;
+			if (verbose > 2) {
+				rprintf(FINFO, "renaming %s to %s\n",
+					safe_fname(partialptr),
+					safe_fname(fname));
+			}
+			if (do_rename(partialptr, fname) < 0) {
+				rsyserr(FERROR, errno,
+					"rename failed for %s (from %s)",
+					full_fname(fname),
+					safe_fname(partialptr));
+			} else {
+				if (remove_sent_files
+				    || (preserve_hard_links
+				     && file->link_u.links)) {
+					SIVAL(numbuf, 0, i);
+					send_msg(MSG_SUCCESS,numbuf,4);
+				}
+				handle_partial_dir(partialptr,
+						   PDIR_DELETE);
+			}
+		}
+	}
+}
+
+static int get_next_gen_i(int batch_gen_fd, int next_gen_i, int desired_i)
+{
+	while (next_gen_i < desired_i) {
+		if (next_gen_i >= 0) {
+			rprintf(FINFO,
+				"(No batched update for%s \"%s\")\n",
+				phase ? " resend of" : "",
+				safe_fname(f_name(the_file_list->files[next_gen_i])));
+		}
+		next_gen_i = read_int(batch_gen_fd);
+		if (next_gen_i == -1)
+			next_gen_i = the_file_list->count;
+	}
+	return next_gen_i;
+}
+
 
 /**
  * main routine for receiver process.
@@ -334,15 +370,20 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 	int next_gen_i = -1;
 	int fd1,fd2;
 	STRUCT_STAT st;
+	int iflags, xlen;
 	char *fname, fbuf[MAXPATHLEN];
-	char template[MAXPATHLEN];
+	char xname[MAXPATHLEN];
 	char fnametmp[MAXPATHLEN];
-	char *fnamecmp, *partialptr;
+	char *fnamecmp, *partialptr, numbuf[4];
 	char fnamecmpbuf[MAXPATHLEN];
+	uchar fnamecmp_type;
 	struct file_struct *file;
 	struct stats initial_stats;
 	int save_make_backups = make_backups;
-	int i, recv_ok, phase = 0;
+	int itemizing = am_daemon ? daemon_log_format_has_i
+		      : !am_server && log_format_has_i;
+	int max_phase = protocol_version >= 29 ? 2 : 1;
+	int i, recv_ok;
 
 	if (verbose > 2)
 		rprintf(FINFO,"recv_files(%d) starting\n",flist->count);
@@ -352,107 +393,150 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 		flist->hlink_pool = NULL;
 	}
 
+	if (delay_updates)
+		init_delayed_bits(flist->count);
+
 	while (1) {
 		cleanup_disable();
 
 		i = read_int(f_in);
 		if (i == -1) {
 			if (read_batch) {
-				if (next_gen_i != flist->count)
-					while (read_int(batch_gen_fd) != -1) {}
+				get_next_gen_i(batch_gen_fd, next_gen_i,
+					       flist->count);
 				next_gen_i = -1;
 			}
-
-			if (phase)
+			if (++phase > max_phase)
 				break;
-
-			phase = 1;
 			csum_length = SUM_LENGTH;
 			if (verbose > 2)
 				rprintf(FINFO, "recv_files phase=%d\n", phase);
+			if (phase == 2 && delay_updates)
+				handle_delayed_updates(flist, local_name);
 			send_msg(MSG_DONE, "", 0);
-			if (keep_partial)
+			if (keep_partial && !partial_dir)
 				make_backups = 0; /* prevents double backup */
 			continue;
 		}
 
-		if (i < 0 || i >= flist->count) {
-			rprintf(FERROR,"Invalid file index %d in recv_files (count=%d)\n",
-				i, flist->count);
-			exit_cleanup(RERR_PROTOCOL);
-		}
+		iflags = read_item_attrs(f_in, -1, i, &fnamecmp_type,
+					 xname, &xlen);
+		if (iflags == ITEM_IS_NEW) /* no-op packet */
+			continue;
 
 		file = flist->files[i];
+		fname = local_name ? local_name : f_name_to(file, fbuf);
+
+		if (verbose > 2)
+			rprintf(FINFO, "recv_files(%s)\n", safe_fname(fname));
+
+		if (!(iflags & ITEM_TRANSFER)) {
+			maybe_log_item(file, iflags, itemizing, xname);
+			continue;
+		}
+		if (phase == 2) {
+			rprintf(FERROR,
+				"got transfer request in phase 2 [%s]\n",
+				who_am_i());
+			exit_cleanup(RERR_PROTOCOL);
+		}
 
 		stats.current_file_index = i;
 		stats.num_transferred_files++;
 		stats.total_transferred_size += file->length;
 		cleanup_got_literal = 0;
 
-		if (local_name)
-			fname = local_name;
-		else
-			fname = f_name_to(file, fbuf);
-
-		if (dry_run) {
-			if (!am_server && verbose) /* log the transfer */
-				rprintf(FINFO, "%s\n", safe_fname(fname));
-			continue;
-		}
-
-		initial_stats = stats;
-
-		if (verbose > 2)
-			rprintf(FINFO, "recv_files(%s)\n", safe_fname(fname));
-
-		if (read_batch) {
-			while (i > next_gen_i) {
-				next_gen_i = read_int(batch_gen_fd);
-				if (next_gen_i == -1)
-					next_gen_i = flist->count;
-			}
-			if (i < next_gen_i) {
-				rprintf(FINFO, "skipping update for \"%s\"\n",
-					safe_fname(fname));
-				discard_receive_data(f_in, file->length);
-				continue;
-			}
-		}
-
-		if (server_exclude_list.head
-		    && check_exclude(&server_exclude_list, fname,
-				     S_ISDIR(file->mode)) < 0) {
+		if (server_filter_list.head
+		    && check_filter(&server_filter_list, fname, 0) < 0) {
 			rprintf(FERROR, "attempt to hack rsync failed.\n");
 			exit_cleanup(RERR_PROTOCOL);
 		}
 
-		if (partial_dir) {
-			if ((partialptr = partial_dir_fname(fname)) != NULL)
+		if (dry_run) { /* log the transfer */
+			if (!am_server && log_format)
+				log_item(file, &stats, iflags, NULL);
+			if (read_batch)
+				discard_receive_data(f_in, file->length);
+			continue;
+		}
+
+		if (read_batch) {
+			next_gen_i = get_next_gen_i(batch_gen_fd, next_gen_i, i);
+			if (i < next_gen_i) {
+				rprintf(FINFO, "(Skipping batched update for \"%s\")\n",
+					safe_fname(fname));
+				discard_receive_data(f_in, file->length);
+				continue;
+			}
+			next_gen_i = -1;
+		}
+
+		partialptr = partial_dir ? partial_dir_fname(fname) : fname;
+
+		if (protocol_version >= 29) {
+			switch (fnamecmp_type) {
+			case FNAMECMP_FNAME:
+				fnamecmp = fname;
+				break;
+			case FNAMECMP_PARTIAL_DIR:
+				fnamecmp = partialptr;
+				break;
+			case FNAMECMP_BACKUP:
+				fnamecmp = get_backup_name(fname);
+				break;
+			case FNAMECMP_FUZZY:
+				if (file->dirname) {
+					pathjoin(fnamecmpbuf, MAXPATHLEN,
+						 file->dirname, xname);
+					fnamecmp = fnamecmpbuf;
+				} else
+					fnamecmp = xname;
+				break;
+			default:
+				if (fnamecmp_type >= basis_dir_cnt) {
+					rprintf(FERROR,
+						"invalid basis_dir index: %d.\n",
+						fnamecmp_type);
+					exit_cleanup(RERR_PROTOCOL);
+				}
+				pathjoin(fnamecmpbuf, sizeof fnamecmpbuf,
+					 basis_dir[fnamecmp_type], fname);
+				fnamecmp = fnamecmpbuf;
+				break;
+			}
+			if (!fnamecmp || (server_filter_list.head
+			  && check_filter(&server_filter_list, fname, 0) < 0))
+				fnamecmp = fname;
+		} else {
+			/* Reminder: --inplace && --partial-dir are never
+			 * enabled at the same time. */
+			if (inplace && make_backups) {
+				if (!(fnamecmp = get_backup_name(fname)))
+					fnamecmp = fname;
+			} else if (partial_dir && partialptr)
 				fnamecmp = partialptr;
 			else
 				fnamecmp = fname;
-		} else
-			fnamecmp = partialptr = fname;
-
-		if (inplace && make_backups) {
-			if (!(fnamecmp = get_backup_name(fname)))
-				fnamecmp = partialptr;
 		}
 
+		initial_stats = stats;
+
 		/* open the file */
 		fd1 = do_open(fnamecmp, O_RDONLY, 0);
 
-		if (fd1 == -1 && fnamecmp != fname) {
-			fnamecmp = fname;
-			fd1 = do_open(fnamecmp, O_RDONLY, 0);
-		}
+		if (fd1 == -1 && protocol_version < 29) {
+			if (fnamecmp != fname) {
+				fnamecmp = fname;
+				fd1 = do_open(fnamecmp, O_RDONLY, 0);
+			}
 
-		if (fd1 == -1 && compare_dest != NULL) {
-			/* try the file at compare_dest instead */
-			pathjoin(fnamecmpbuf, sizeof fnamecmpbuf,
-				 compare_dest, fname);
-			fnamecmp = fnamecmpbuf;
-			fd1 = do_open(fnamecmp, O_RDONLY, 0);
+			if (fd1 == -1 && basis_dir[0]) {
+				/* pre-29 allowed only one alternate basis */
+				pathjoin(fnamecmpbuf, sizeof fnamecmpbuf,
+					 basis_dir[0], fname);
+				fnamecmp = fnamecmpbuf;
+				fd1 = do_open(fnamecmp, O_RDONLY, 0);
+			}
 		}
 
 		if (fd1 != -1 && do_fstat(fd1,&st) != 0) {
@@ -507,8 +591,6 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 				continue;
 			}
 
-			strlcpy(template, fnametmp, sizeof template);
-
 			/* we initially set the perms without the
 			 * setuid/setgid bits to ensure that there is no race
 			 * condition. They are then correctly updated after
@@ -522,7 +604,8 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 			 * transferred, but that may not be the case with -R */
 			if (fd2 == -1 && relative_paths && errno == ENOENT
 			    && create_directory_path(fnametmp, orig_umask) == 0) {
-				strlcpy(fnametmp, template, sizeof fnametmp);
+				/* Get back to name with XXXXXX in it. */
+				get_tmpname(fnametmp, fname);
 				fd2 = do_mkstemp(fnametmp, file->mode & INITACCESSPERMS);
 			}
 			if (fd2 == -1) {
@@ -538,14 +621,18 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 				cleanup_set(fnametmp, partialptr, file, fd1, fd2);
 		}
 
-		if (!am_server && verbose) /* log the transfer */
+		/* log the transfer */
+		if (log_before_transfer)
+			log_item(file, &initial_stats, iflags, NULL);
+		else if (!am_server && verbose && do_progress)
 			rprintf(FINFO, "%s\n", safe_fname(fname));
 
 		/* recv file data */
 		recv_ok = receive_data(f_in, fnamecmp, fd1, st.st_size,
 				       fname, fd2, file->length);
 
-		log_recv(file, &initial_stats);
+		if (!log_before_transfer)
+			log_item(file, &initial_stats, iflags, NULL);
 
 		if (fd1 != -1)
 			close(fd1);
@@ -555,26 +642,35 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 			exit_cleanup(RERR_FILEIO);
 		}
 
-		if (recv_ok || inplace)
-			finish_transfer(fname, fnametmp, file, recv_ok);
-		else if (keep_partial && partialptr
-		    && handle_partial_dir(partialptr, PDIR_CREATE))
-			finish_transfer(partialptr, fnametmp, file, 0);
-		else {
+		if ((recv_ok && (!delay_updates || !partialptr)) || inplace) {
+			finish_transfer(fname, fnametmp, file, recv_ok, 1);
+			if (partialptr != fname && fnamecmp == partialptr) {
+				do_unlink(partialptr);
+				handle_partial_dir(partialptr, PDIR_DELETE);
+			}
+		} else if (keep_partial && partialptr
+		    && handle_partial_dir(partialptr, PDIR_CREATE)) {
+			finish_transfer(partialptr, fnametmp, file, recv_ok,
+					!partial_dir);
+			if (delay_updates && recv_ok) {
+				set_delayed_bit(i);
+				recv_ok = -1;
+			}
+		} else {
 			partialptr = NULL;
 			do_unlink(fnametmp);
 		}
 
-		if (partialptr != fname && fnamecmp == partialptr && recv_ok) {
-			do_unlink(partialptr);
-			handle_partial_dir(partialptr, PDIR_DELETE);
-		}
-
 		cleanup_disable();
 
-		if (!recv_ok) {
-			int msgtype = csum_length == SUM_LENGTH || read_batch ?
-				FERROR : FINFO;
+		if (recv_ok > 0) {
+			if (remove_sent_files
+			    || (preserve_hard_links && file->link_u.links)) {
+				SIVAL(numbuf, 0, i);
+				send_msg(MSG_SUCCESS, numbuf, 4);
+			}
+		} else if (!recv_ok) {
+			int msgtype = phase || read_batch ? FERROR : FINFO;
 			if (msgtype == FERROR || verbose) {
 				char *errstr, *redostr, *keptstr;
 				if (!(keep_partial && partialptr) && !inplace)
@@ -595,17 +691,16 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 					errstr, safe_fname(fname),
 					keptstr, redostr);
 			}
-			if (csum_length != SUM_LENGTH) {
-				char buf[4];
-				SIVAL(buf, 0, i);
-				send_msg(MSG_REDO, buf, 4);
+			if (!phase) {
+				SIVAL(numbuf, 0, i);
+				send_msg(MSG_REDO, numbuf, 4);
 			}
 		}
 	}
 	make_backups = save_make_backups;
 
-	if (delete_after && recurse && !local_name && flist->count > 0)
-		delete_files(flist);
+	if (phase == 2 && delay_updates) /* for protocol_version < 29 */
+		handle_delayed_updates(flist, local_name);
 
 	if (verbose > 2)
 		rprintf(FINFO,"recv_files finished\n");
