@@ -26,6 +26,10 @@
 
 #include "rsync.h"
 
+#if !defined MKNOD_CREATES_SOCKETS && defined HAVE_SYS_UN_H
+#include <sys/un.h>
+#endif
+
 extern int dry_run;
 extern int read_only;
 extern int list_only;
@@ -55,7 +59,7 @@ int do_symlink(char *fname1, char *fname2)
 	return symlink(fname1, fname2);
 }
 
-#if HAVE_LINK
+#ifdef HAVE_LINK
 int do_link(char *fname1, char *fname2)
 {
 	if (dry_run) return 0;
@@ -68,17 +72,47 @@ int do_lchown(const char *path, uid_t owner, gid_t group)
 {
 	if (dry_run) return 0;
 	RETURN_ERROR_IF_RO_OR_LO;
+#ifndef HAVE_LCHOWN
+#define lchown chown
+#endif
 	return lchown(path, owner, group);
 }
 
-#if HAVE_MKNOD
 int do_mknod(char *pathname, mode_t mode, dev_t dev)
 {
 	if (dry_run) return 0;
 	RETURN_ERROR_IF_RO_OR_LO;
-	return mknod(pathname, mode, dev);
-}
+#if !defined MKNOD_CREATES_FIFOS && defined HAVE_MKFIFO
+	if (S_ISFIFO(mode))
+		return mkfifo(pathname, mode);
 #endif
+#if !defined MKNOD_CREATES_SOCKETS && defined HAVE_SYS_UN_H
+	if (S_ISSOCK(mode)) {
+		int sock;
+		struct sockaddr_un saddr;
+		unsigned int len;
+
+		saddr.sun_family = AF_UNIX;
+		len = strlcpy(saddr.sun_path, pathname, sizeof saddr.sun_path);
+#ifdef HAVE_SOCKADDR_UN_LEN
+		saddr.sun_len = len >= sizeof saddr.sun_path
+			      ? sizeof saddr.sun_path : len + 1;
+#endif
+
+		if ((sock = socket(PF_UNIX, SOCK_STREAM, 0)) < 0
+		    || (unlink(pathname) < 0 && errno != ENOENT)
+		    || (bind(sock, (struct sockaddr*)&saddr, sizeof saddr)) < 0)
+			return -1;
+		close(sock);
+		return do_chmod(pathname, mode);
+	}
+#endif
+#ifdef HAVE_MKNOD
+	return mknod(pathname, mode, dev);
+#else
+	return -1;
+#endif
+}
 
 int do_rmdir(char *pathname)
 {
@@ -97,7 +131,7 @@ int do_open(char *pathname, int flags, mode_t mode)
 	return open(pathname, flags | O_BINARY, mode);
 }
 
-#if HAVE_CHMOD
+#ifdef HAVE_CHMOD
 int do_chmod(const char *path, mode_t mode)
 {
 	int code;
@@ -152,7 +186,7 @@ int do_mkstemp(char *template, mode_t perms)
 	RETURN_ERROR_IF(dry_run, 0);
 	RETURN_ERROR_IF(read_only, EROFS);
 
-#if HAVE_SECURE_MKSTEMP && HAVE_FCHMOD && (!HAVE_OPEN64 || HAVE_MKSTEMP64)
+#if defined HAVE_SECURE_MKSTEMP && defined HAVE_FCHMOD && (!defined HAVE_OPEN64 || defined HAVE_MKSTEMP64)
 	{
 		int fd = mkstemp(template);
 		if (fd == -1)
@@ -164,6 +198,9 @@ int do_mkstemp(char *template, mode_t perms)
 			errno = errno_save;
 			return -1;
 		}
+#if defined HAVE_SETMODE && O_BINARY
+		setmode(fd, O_BINARY);
+#endif
 		return fd;
 	}
 #else
@@ -175,27 +212,29 @@ int do_mkstemp(char *template, mode_t perms)
 
 int do_stat(const char *fname, STRUCT_STAT *st)
 {
-#if HAVE_OFF64_T
+#ifdef USE_STAT64_FUNCS
 	return stat64(fname, st);
 #else
 	return stat(fname, st);
 #endif
 }
 
-#if SUPPORT_LINKS
 int do_lstat(const char *fname, STRUCT_STAT *st)
 {
-#if HAVE_OFF64_T
+#ifdef SUPPORT_LINKS
+# ifdef USE_STAT64_FUNCS
 	return lstat64(fname, st);
-#else
+# else
 	return lstat(fname, st);
+# endif
+#else
+	return do_stat(fname, st);
 #endif
 }
-#endif
 
 int do_fstat(int fd, STRUCT_STAT *st)
 {
-#if HAVE_OFF64_T
+#ifdef USE_STAT64_FUNCS
 	return fstat64(fd, st);
 #else
 	return fstat(fd, st);
@@ -204,7 +243,7 @@ int do_fstat(int fd, STRUCT_STAT *st)
 
 OFF_T do_lseek(int fd, OFF_T offset, int whence)
 {
-#if HAVE_OFF64_T
+#if SIZEOF_OFF64_T
 	off64_t lseek64();
 	return lseek64(fd, offset, whence);
 #else
@@ -212,20 +251,9 @@ OFF_T do_lseek(int fd, OFF_T offset, int whence)
 #endif
 }
 
-#ifdef USE_MMAP
-void *do_mmap(void *start, int len, int prot, int flags, int fd, OFF_T offset)
-{
-#if HAVE_OFF64_T
-	return mmap64(start, len, prot, flags, fd, offset);
-#else
-	return mmap(start, len, prot, flags, fd, offset);
-#endif
-}
-#endif
-
 char *d_name(struct dirent *di)
 {
-#if HAVE_BROKEN_READDIR
+#ifdef HAVE_BROKEN_READDIR
 	return (di->d_name - 2);
 #else
 	return di->d_name;
